; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; Nom du projet : Dev-Module
; Nom du fichier : DevModule - DevModule.pb
; Version du fichier : 0.9.10
; Programmation : En cours
; Programm par : Guillaume Saumure
; Alias : StarBootics
; Courriel : gsaumure@cgocable.ca
; Date : 03-10-2014
; Mise  jour : 08-02-2019
; Cod pour PureBasic : V5.70 LTS
; Plateforme : Windows, Linux, MacOS X
; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

DeclareModule DevModule
  
  Declare Initialize()
  Declare CreateWindow()
  Declare EventManager()
  
EndDeclareModule

Module DevModule

  IncludeFile "DevModule - Constants.pb"
  IncludeFile "DevModule - Language.pb"
  IncludeFile "DevModule - Extra.pb"
  
  Global PrefFileName.s
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< OpenDevModulePreferences (Private) <<<<<<
  
  Procedure OpenDevModulePreferences()
    
    If OpenPreferences(PrefFileName)
      
      Window::ReadPrefs("MainWin", #MainWin)
      Window::ReadPrefs("InternalStructureWin", #InternalStructureWin)
      Window::ReadPrefs("OptionsWin", #OptionsWin)
      Language::ReadPrefs("Language")
      Options::ReadPrefs("Options")
      Sticky::ReadPrefs("Sticky")
      Comments::ReadPrefs("Comments")
      General::ReadPrefs("CodeGenerator")
      SyntaxHighlighting::ReadPrefs("SyntaxHighlighting")
      Stats::ReadPrefs("Statistics")
      
      ClosePreferences()
      
    EndIf
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< CreateDevModulePreferences (Private) <<<<<<
  
  Procedure CreateDevModulePreferences()
    
    If CreatePreferences(PrefFileName)
      
      Window::WritePrefs("MainWin", #MainWin)
      Window::WritePrefs("InternalStructureWin", #InternalStructureWin)
      Window::WritePrefs("OptionsWin", #OptionsWin)
      Language::WritePrefs("Language")
      Options::WritePrefs("Options")
      Sticky::WritePrefs("Sticky")
      Comments::WritePrefs("Comments")
      General::WritePrefs("CodeGenerator")
      SyntaxHighlighting::WritePrefs("SyntaxHighlighting")
      Stats::WritePrefs("Statistics")
      
      ClosePreferences()
      
    EndIf
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< ShutDown (Private) <<<<<
  
  Procedure ShutDown()
    
    Window::GetCurrentPositionSize(#MainWin)
    
    CreateDevModulePreferences()
    
    For GadgetID = #CheckBox_Observator To #CheckBox_ReadBinary Step 3
      UnbindGadgetEvent(GadgetID, @MethodGadgetFollowCheckBox())
    Next
    
    UnbindGadgetEvent(#TrackBar_Tabulation_Size, @TextFollowTrackBar()) 
    UnbindGadgetEvent(#Option_Use_NBSPACE_Tabulation, @TrackBarFollowOption())
    
    RemoveKeyboardShortcut(#MainWin, #PB_Shortcut_All)
    
    GoScintilla::Free(#Editor_Module_Primary)
    GoScintilla::Free(#Editor_Module_Secondary)
    GoScintilla::Free(#Editor_Source_Code)
    
    CloseWindow(#MainWin)
    
    Language::Reset()
    Window::Reset(-1)
    Sticky::Reset()
    Options::Reset()
    Field::Reset(Field::#RESET_STYLE_TERMINAL)
    MatrixGenerator::Reset()
    Comments::Reset()
    AdvancedDateFormatting::Reset()
    AskPureBasicCompiler::Reset()
    Stats::Reset()
    Chrono::Reset(-1)
    
    PrefFileName = ""
    End
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< PastedCodeCleanUp (Private) <<<<<
  
  Procedure PastedCodeCleanUp(GadgetID)
    
    Code.s = GoScintilla::GetText(GadgetID)
    GoScintilla::ClearText(GadgetID)
    
    LineMax = CountString(Code, LineFeed::Single()) + 1
    
    For LineID = 1 To LineMax
      
      Field.s = Trim(RemoveString(StringField(Code, LineID, LineFeed::Single()), Chr(9)))
      
      If Field <> ""
        
        If LineID = LineMax
          GoScintilla::AppendText(GadgetID, Field)
        Else
          GoScintilla::AppendText(GadgetID, Field + LineFeed::Single())
        EndIf
        
      EndIf
      
    Next 
    
  EndProcedure

  Procedure From_GUI()
    
    If Trim(GetGadgetText(#String_Module_Name)) = ""
      MessageRequester(Language::Message("Dev-Module - Error"), Language::Message("The Module name is missing !"))
      ProcedureReturn #False
    EndIf
    
    If Comments::DateMaskSize() = 0
      MessageRequester(Language::Message("Dev-Module - Error"), Language::Message("No Date Mask !"))
      ProcedureReturn #False
    EndIf
    
    General::SetModuleName(Trim(GetGadgetText(#String_Module_Name)))
    
    If GetGadgetState(#Option_Single_Instance) = 1
      General::SetInstanceStyle(Field::#INSTANCE_STYLE_UNIQUE)
    ElseIf GetGadgetState(#Option_Multiple_Instance_ID) = 1
      General::SetInstanceStyle(Field::#INSTANCE_STYLE_MULTIPLE_ID)
    ElseIf GetGadgetState(#Option_Multiple_Instance_Index) = 1
      General::SetInstanceStyle(Field::#INSTANCE_STYLE_MULTIPLE_INDEX)
    ElseIf GetGadgetState(#Option_Multiple_Instance_Name) = 1
      General::SetInstanceStyle(Field::#INSTANCE_STYLE_MULTIPLE_NAME)
    ElseIf GetGadgetState(#Option_Off_Module_Instance) = 1
      General::SetInstanceStyle(Field::#INSTANCE_STYLE_OFF_MODULE)
    EndIf

    PastedCodeCleanUp(#Editor_Module_Primary)
    PastedCodeCleanUp(#Editor_Module_Secondary)
    
    If General::GetInstanceStyle() = Field::#INSTANCE_STYLE_MULTIPLE_ID
      
      Content.s = GoScintilla::GetText(#Editor_Module_Primary)
      Field_Max = CountString(Content, LineFeed::Single()) + 1
      
      For FieldID = 1 To Field_Max
        
        If StringField(Content, FieldID, LineFeed::Single()) = "ID.l"
          ID_Field_Found = #True
          Break
        EndIf
        
      Next
      
      If ID_Field_Found = #False
        GoScintilla::SetText(#Editor_Module_Primary, "ID.l" + LineFeed::Single())
        GoScintilla::AppendText(#Editor_Module_Primary, Content)
      EndIf
      
    EndIf
    
    Content.s = GoScintilla::GetText(#Editor_Module_Primary)
    Field_Max = CountString(Content, LineFeed::Single()) + 1
    
    For FieldID = 1 To Field_Max
      
      Definition.s = StringField(Content, FieldID, LineFeed::Single())
      
      If Definition <> ""
        Field::New(0, FieldID, Definition)
      EndIf
      
    Next
    
    Content.s = GoScintilla::GetText(#Editor_Module_Secondary)
    Field_Max = CountString(Content, LineFeed::Single()) + 1
   
    For FieldID = 1 To Field_Max
      
      Definition.s = StringField(Content, FieldID, LineFeed::Single())
      
      If Definition <> ""
        Field::New(1, FieldID, Definition)
      EndIf
      
    Next
    
    If Field::InstanceSize(1) >= 1
      General::SetSecondaryInstance(#True)
    EndIf
    
    MethodGadgetsReverseStatus(BasicMethods::@SetObservator(), #CheckBox_Observator)
    MethodGadgetsReverseStatus(BasicMethods::@SetMutator(), #CheckBox_Mutator)
    MethodGadgetsReverseStatus(BasicMethods::@SetUpdate(), #CheckBox_Update)
    MethodGadgetsReverseStatus(BasicMethods::@SetInitialize(), #CheckBox_Initialize)
    MethodGadgetsReverseStatus(BasicMethods::@SetReset(), #CheckBox_Reset)
    MethodGadgetsReverseStatus(BasicMethods::@SetClear(), #CheckBox_Clear)
    
    MethodGadgetsReverseStatus(StaticArrayMethods::@SetQuickSortAscendant(), #CheckBox_QuickSortAscending)
    MethodGadgetsReverseStatus(StaticArrayMethods::@SetQuickSortDescendant(), #CheckBox_QuickSortDescending)
    MethodGadgetsReverseStatus(StaticArrayMethods::@SetLinearSearch(), #CheckBox_LinearSearch)
    MethodGadgetsReverseStatus(StaticArrayMethods::@SetBinarySearch(), #CheckBox_BinarySearch)
    MethodGadgetsReverseStatus(StaticArrayMethods::@SetFixedLenghtQueue(), #CheckBox_FixedLengthQueue)
    
    MethodGadgetsReverseStatus(LinkedListMethods::@SetSort(), #CheckBox_Sort_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetLinearSearch(), #CheckBox_Lin_Search_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetBinarySearch(), #CheckBox_Bin_Search_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetAdd(), #CheckBox_Add_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetSelect(), #CheckBox_Select_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetFirst(), #CheckBox_First_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetLast(), #CheckBox_Last_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetPrevious(), #CheckBox_Previous_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetNext(), #CheckBox_Next_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetDelete(), #CheckBox_Delete_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetReset(), #CheckBox_Reset_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetClear(), #CheckBox_Clear_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetIndex(), #CheckBox_Index_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetSize(), #CheckBox_Size_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetInsert(), #CheckBox_Insert_Linked_List)
    MethodGadgetsReverseStatus(LinkedListMethods::@SetExtract(), #CheckBox_Extract_Linked_List)
    
    MethodGadgetsReverseStatus(DynamicArrayMethods::@SetSort(), #CheckBox_Sort_Dynamic_Array)
    MethodGadgetsReverseStatus(DynamicArrayMethods::@SetLinearSearch(), #CheckBox_Lin_Search_Dynamic_Array)
    MethodGadgetsReverseStatus(DynamicArrayMethods::@SetBinarySearch(), #CheckBox_Bin_Search_Dynamic_Array)
    MethodGadgetsReverseStatus(DynamicArrayMethods::@SetInsert(), #CheckBox_Insert_Dynamic_Array)
    MethodGadgetsReverseStatus(DynamicArrayMethods::@SetExtract(), #CheckBox_Extract_Dynamic_Array)
    
    MethodGadgetsReverseStatus(MapMethods::@SetAdd(), #CheckBox_Add_Map)
    MethodGadgetsReverseStatus(MapMethods::@SetClear(), #CheckBox_Clear_Map)
    MethodGadgetsReverseStatus(MapMethods::@SetSize(), #CheckBox_Size_Map)
    MethodGadgetsReverseStatus(MapMethods::@SetReset(), #CheckBox_Reset_Map)
    MethodGadgetsReverseStatus(MapMethods::@SetDelete(), #CheckBox_Delete_Map)
    MethodGadgetsReverseStatus(MapMethods::@SetFind(), #CheckBox_Find_Map)
    MethodGadgetsReverseStatus(MapMethods::@SetNext(), #CheckBox_Next_Map)
    MethodGadgetsReverseStatus(MapMethods::@SetKey(), #CheckBox_Key_Map)
    MethodGadgetsReverseStatus(MapMethods::@SetInsert(), #CheckBox_Insert_Map)
    MethodGadgetsReverseStatus(MapMethods::@SetExtract(), #CheckBox_Extract_Map)
    
    MethodGadgetsReverseStatus(MathsMethods::@SetEqual(), #CheckBox_Equal)
    MethodGadgetsReverseStatus(MathsMethods::@SetPlus(), #CheckBox_Plus)
    MethodGadgetsReverseStatus(MathsMethods::@SetMinus(), #CheckBox_Minus)
    MethodGadgetsReverseStatus(MathsMethods::@SetProduct(), #CheckBox_Product)
    MethodGadgetsReverseStatus(MathsMethods::@SetDivide(), #CheckBox_Divide)
    MethodGadgetsReverseStatus(MathsMethods::@SetProductByScalar(), #CheckBox_ProductByScalar)
    MethodGadgetsReverseStatus(MathsMethods::@SetDivideByScalar(), #CheckBox_DivideByScalar)
    MethodGadgetsReverseStatus(MathsMethods::@SetCopy(), #CheckBox_Copy)
    MethodGadgetsReverseStatus(MathsMethods::@SetCompare(), #CheckBox_Compare)
    MethodGadgetsReverseStatus(MathsMethods::@SetSwapping(), #CheckBox_Swapping)

    MethodGadgetsReverseStatus(XMLFileMethods::@SetNodeWithChildNode(), #CheckBox_XML_SetGet_Node)
    MethodGadgetsReverseStatus(XMLFileMethods::@SetNodeWithAttributes(), #CheckBox_XML_SetGet_Attribute)
    MethodGadgetsReverseStatus(XMLFileMethods::@SetAddCollectAttributes(), #CheckBox_XML_AddCollect_Attribute)
    MethodGadgetsReverseStatus(XMLFileMethods::@SetCreateLoadFile(), #CheckBox_XML_CreateLoad_File)
    MethodGadgetsReverseStatus(XMLFileMethods::@SetCatchFile(), #CheckBox_XML_Catch_File)
    
    MethodGadgetsReverseStatus(JSONFileMethods::@SetCreateLoadFile(), #CheckBox_JSON_CreateLoad_File)
    
    MethodGadgetsReverseStatus(DataBaseMethods::@SetCreateTable(), #CheckBox_DataBase_CreateTable)
    MethodGadgetsReverseStatus(DataBaseMethods::@SetDropTable(), #CheckBox_DataBase_DropTable)
    MethodGadgetsReverseStatus(DataBaseMethods::@SetTruncateTable(), #CheckBox_DataBase_TruncateTable)
    MethodGadgetsReverseStatus(DataBaseMethods::@SetSelectTable(), #CheckBox_DataBase_SelectTable)
    MethodGadgetsReverseStatus(DataBaseMethods::@SetCountRow(), #CheckBox_DataBase_CountRowInTable)
    MethodGadgetsReverseStatus(DataBaseMethods::@SetOpenFile(), #CheckBox_DataBase_OpenFile)
    MethodGadgetsReverseStatus(DataBaseMethods::@SetSendModuleTo(), #CheckBox_DataBase_ModuleTo)
    MethodGadgetsReverseStatus(DataBaseMethods::@SetLoadModuleFrom(), #CheckBox_DataBase_ToModule)
    MethodGadgetsReverseStatus(DataBaseMethods::@SetVacummize(), #CheckBox_DataBase_Vacuumize)

    If GetGadgetState(#CheckBox_DataBase_OpenFile) Or GetGadgetState(#CheckBox_DataBase_ModuleTo) Or GetGadgetState(#CheckBox_DataBase_ToModule)

      If General::GetInstanceStyle() = Field::#INSTANCE_STYLE_UNIQUE
        General::SetIsDatabaseHelperNeeded(1) 
      ElseIf General::GetInstanceStyle() = Field::#INSTANCE_STYLE_MULTIPLE_ID
        General::SetIsDatabaseHelperNeeded(2)  
      ElseIf General::GetInstanceStyle() = Field::#INSTANCE_STYLE_MULTIPLE_INDEX
        General::SetIsDatabaseHelperNeeded(2)  
      ElseIf General::GetInstanceStyle() = Field::#INSTANCE_STYLE_MULTIPLE_NAME
        General::SetIsDatabaseHelperNeeded(2)
      Else
        General::SetIsDatabaseHelperNeeded(0)  
      EndIf

    EndIf

    MethodGadgetsReverseStatus(PrefsFileMethods::@SetReadWriteGroup(), #CheckBox_Pref_Group)
    MethodGadgetsReverseStatus(PrefsFileMethods::@SetCreateOpenFile(), #CheckBox_Pref_File)
    
    MethodGadgetsReverseStatus(BinaryFileMethods::@SetReadWrite(), #CheckBox_ReadWriteBinary)
    MethodGadgetsReverseStatus(BinaryFileMethods::@SetCreateFile(), #CheckBox_CreateBinary)
    MethodGadgetsReverseStatus(BinaryFileMethods::@SetOpenFile(), #CheckBox_OpenBinary)
    MethodGadgetsReverseStatus(BinaryFileMethods::@SetReadFile(), #CheckBox_ReadBinary)
    
    MethodGadgetsReverseStatus(EncodedFileMethods::@SetReadWrite(), #CheckBox_ReadWriteEncoded)
    MethodGadgetsReverseStatus(EncodedFileMethods::@SetCreateFile(), #CheckBox_CreateEncoded)
    MethodGadgetsReverseStatus(EncodedFileMethods::@SetOpenFile(), #CheckBox_OpenEncoded)
    MethodGadgetsReverseStatus(EncodedFileMethods::@SetReadFile(), #CheckBox_ReadEncoded)
    
    EncodedFileMethods::SetExternalModuleToUse(GetGadgetText(#ComboBox_EncodedFile_Module)) 
    General::CurrentFileModuleName(0, EncodedFileMethods::GetExternalModuleToUse())
    
    MethodGadgetsReverseStatus(CryptedFileMethods::@SetReadWrite(), #CheckBox_ReadWriteCrypted)
    MethodGadgetsReverseStatus(CryptedFileMethods::@SetCreateFile(), #CheckBox_CreateCrypted)
    MethodGadgetsReverseStatus(CryptedFileMethods::@SetOpenFile(), #CheckBox_OpenCrypted)
    MethodGadgetsReverseStatus(CryptedFileMethods::@SetReadFile(), #CheckBox_ReadCrypted)

    CryptedFileMethods::SetExternalModuleToUse(GetGadgetText(#ComboBox_CryptedFile_Module))
    General::CurrentFileModuleName(1, CryptedFileMethods::GetExternalModuleToUse())
    
    MethodGadgetsReverseStatus(DebuggingMethods::@SetDebugging(), #CheckBox_Debugging)
    
    ProcedureReturn #True
  EndProcedure

  Procedure To_GUI()
    
    GoScintilla::ClearText(#Editor_Module_Primary)
    GoScintilla::ClearText(#Editor_Module_Secondary)
    GoScintilla::ClearText(#Editor_Source_Code)
    ClearGadgetItems(#ListView_Internal_Structure)
    
    SetGadgetText(#String_Module_Name, General::GetModuleName())

    If General::GetInstanceStyle() = Field::#INSTANCE_STYLE_UNIQUE
      SetGadgetState(#Option_Single_Instance, 1)
    ElseIf General::GetInstanceStyle() = Field::#INSTANCE_STYLE_MULTIPLE_ID
      SetGadgetState(#Option_Multiple_Instance_ID, 1)
    ElseIf General::GetInstanceStyle() = Field::#INSTANCE_STYLE_MULTIPLE_INDEX
      SetGadgetState(#Option_Multiple_Instance_Index, 1)
    ElseIf General::GetInstanceStyle() = Field::#INSTANCE_STYLE_MULTIPLE_NAME
      SetGadgetState(#Option_Multiple_Instance_Name, 1)
    ElseIf General::GetInstanceStyle() = Field::#INSTANCE_STYLE_OFF_MODULE  
      SetGadgetState(#Option_Off_Module_Instance, 1)
    EndIf
    
    If Field::FirstInstance(0)
      
      Repeat 
        
        If Field::InstanceIndex(0) = Field::InstanceSize(0) - 1
          GoScintilla::AppendText(#Editor_Module_Primary, Field::GetFieldDescription(0))
        Else
          GoScintilla::AppendText(#Editor_Module_Primary, Field::GetFieldDescription(0) + LineFeed::Single())
        EndIf
        
      Until Field::NextInstance(0) = #Null
      
    EndIf 
    
    If Field::FirstInstance(1)
      
      Repeat 
        If Field::InstanceIndex(1) = Field::InstanceSize(1) - 1
          GoScintilla::AppendText(#Editor_Module_Secondary, Field::GetFieldDescription(1))
        Else
          GoScintilla::AppendText(#Editor_Module_Secondary, Field::GetFieldDescription(1) + LineFeed::Single())
        EndIf
      Until Field::NextInstance(1) = #Null
      
    EndIf

    If General::FirstStruct()

      Repeat
        AddGadgetItem(#ListView_Internal_Structure, -1, General::GetStructName())
      Until General::NextStruct() = #Null
      
    EndIf
  
    MethodGadgetsStatus(BasicMethods::GetObservator(), #CheckBox_Observator)
    MethodGadgetsStatus(BasicMethods::GetMutator(), #CheckBox_Mutator)
    MethodGadgetsStatus(BasicMethods::GetUpdate(), #CheckBox_Update)
    MethodGadgetsStatus(BasicMethods::GetInitialize(), #CheckBox_Initialize)
    MethodGadgetsStatus(BasicMethods::GetReset(), #CheckBox_Reset)
    MethodGadgetsStatus(BasicMethods::GetClear(), #CheckBox_Clear)
    
    MethodGadgetsStatus(StaticArrayMethods::GetQuickSortAscendant(), #CheckBox_QuickSortAscending)
    MethodGadgetsStatus(StaticArrayMethods::GetQuickSortDescendant(), #CheckBox_QuickSortDescending)
    MethodGadgetsStatus(StaticArrayMethods::GetLinearSearch(), #CheckBox_LinearSearch)
    MethodGadgetsStatus(StaticArrayMethods::GetBinarySearch(), #CheckBox_BinarySearch)
    MethodGadgetsStatus(StaticArrayMethods::GetFixedLenghtQueue(), #CheckBox_FixedLengthQueue)
    
    MethodGadgetsStatus(LinkedListMethods::GetSort(), #CheckBox_Sort_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetLinearSearch(), #CheckBox_Lin_Search_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetBinarySearch(), #CheckBox_Bin_Search_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetAdd(), #CheckBox_Add_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetSelect(), #CheckBox_Select_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetFirst(), #CheckBox_First_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetLast(), #CheckBox_Last_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetPrevious(), #CheckBox_Previous_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetNext(), #CheckBox_Next_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetDelete(), #CheckBox_Delete_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetReset(), #CheckBox_Reset_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetClear(), #CheckBox_Clear_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetIndex(), #CheckBox_Index_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetSize(), #CheckBox_Size_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetInsert(), #CheckBox_Insert_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetExtract(), #CheckBox_Extract_Linked_List)
    
    MethodGadgetsStatus(DynamicArrayMethods::GetSort(), #CheckBox_Sort_Dynamic_Array)
    MethodGadgetsStatus(DynamicArrayMethods::GetLinearSearch(), #CheckBox_Lin_Search_Dynamic_Array)
    MethodGadgetsStatus(DynamicArrayMethods::GetBinarySearch(), #CheckBox_Bin_Search_Dynamic_Array)
    MethodGadgetsStatus(DynamicArrayMethods::GetInsert(), #CheckBox_Insert_Dynamic_Array)
    MethodGadgetsStatus(DynamicArrayMethods::GetExtract(), #CheckBox_Extract_Dynamic_Array)
    
    MethodGadgetsStatus(MapMethods::GetAdd(), #CheckBox_Add_Map)
    MethodGadgetsStatus(MapMethods::GetClear(), #CheckBox_Clear_Map)
    MethodGadgetsStatus(MapMethods::GetSize(), #CheckBox_Size_Map)
    MethodGadgetsStatus(MapMethods::GetReset(), #CheckBox_Reset_Map)
    MethodGadgetsStatus(MapMethods::GetDelete(), #CheckBox_Delete_Map)
    MethodGadgetsStatus(MapMethods::GetFind(), #CheckBox_Find_Map)
    MethodGadgetsStatus(MapMethods::GetNext(), #CheckBox_Next_Map)
    MethodGadgetsStatus(MapMethods::GetKey(), #CheckBox_Key_Map)
    MethodGadgetsStatus(MapMethods::GetInsert(), #CheckBox_Insert_Map)
    MethodGadgetsStatus(MapMethods::GetExtract(), #CheckBox_Extract_Map)
    
    MethodGadgetsStatus(MathsMethods::GetEqual(), #CheckBox_Equal)
    MethodGadgetsStatus(MathsMethods::GetPlus(), #CheckBox_Plus)
    MethodGadgetsStatus(MathsMethods::GetMinus(), #CheckBox_Minus)
    MethodGadgetsStatus(MathsMethods::GetProduct(), #CheckBox_Product)
    MethodGadgetsStatus(MathsMethods::GetDivide(), #CheckBox_Divide)
    MethodGadgetsStatus(MathsMethods::GetProductByScalar(), #CheckBox_ProductByScalar)
    MethodGadgetsStatus(MathsMethods::GetDivideByScalar(), #CheckBox_DivideByScalar)
    MethodGadgetsStatus(MathsMethods::GetCopy(), #CheckBox_Copy)
    MethodGadgetsStatus(MathsMethods::GetCompare(), #CheckBox_Compare)
    MethodGadgetsStatus(MathsMethods::GetSwapping(), #CheckBox_Swapping)
    
    MethodGadgetsStatus(XMLFileMethods::GetNodeWithChildNode(), #CheckBox_XML_SetGet_Node)
    MethodGadgetsStatus(XMLFileMethods::GetNodeWithAttributes(), #CheckBox_XML_SetGet_Attribute)
    MethodGadgetsStatus(XMLFileMethods::GetAddCollectAttributes(), #CheckBox_XML_AddCollect_Attribute)
    MethodGadgetsStatus(XMLFileMethods::GetCreateLoadFile(), #CheckBox_XML_CreateLoad_File)
    MethodGadgetsStatus(XMLFileMethods::GetCatchFile(), #CheckBox_XML_Catch_File)
    
    MethodGadgetsStatus(JSONFileMethods::GetCreateLoadFile(), #CheckBox_JSON_CreateLoad_File)
    
    MethodGadgetsStatus(DataBaseMethods::GetCreateTable(), #CheckBox_DataBase_CreateTable)
    MethodGadgetsStatus(DataBaseMethods::GetDropTable(), #CheckBox_DataBase_DropTable)
    MethodGadgetsStatus(DataBaseMethods::GetTruncateTable(), #CheckBox_DataBase_TruncateTable)
    MethodGadgetsStatus(DataBaseMethods::GetSelectTable(), #CheckBox_DataBase_SelectTable)
    MethodGadgetsStatus(DataBaseMethods::GetCountRow(), #CheckBox_DataBase_CountRowInTable)
    MethodGadgetsStatus(DataBaseMethods::GetOpenFile(), #CheckBox_DataBase_OpenFile)
    MethodGadgetsStatus(DataBaseMethods::GetSendModuleTo(), #CheckBox_DataBase_ModuleTo)
    MethodGadgetsStatus(DataBaseMethods::GetLoadModuleFrom(), #CheckBox_DataBase_ToModule)
    MethodGadgetsStatus(DataBaseMethods::GetVacummize(), #CheckBox_DataBase_Vacuumize)
    
    MethodGadgetsStatus(PrefsFileMethods::GetReadWriteGroup(), #CheckBox_Pref_Group)
    MethodGadgetsStatus(PrefsFileMethods::GetCreateOpenFile(), #CheckBox_Pref_File)
    
    MethodGadgetsStatus(BinaryFileMethods::GetReadWrite(), #CheckBox_ReadWriteBinary)
    MethodGadgetsStatus(BinaryFileMethods::GetCreateFile(), #CheckBox_CreateBinary)
    MethodGadgetsStatus(BinaryFileMethods::GetOpenFile(), #CheckBox_OpenBinary)
    MethodGadgetsStatus(BinaryFileMethods::GetReadFile(), #CheckBox_ReadBinary)
    
    MethodGadgetsStatus(EncodedFileMethods::GetReadWrite(), #CheckBox_ReadWriteEncoded)
    MethodGadgetsStatus(EncodedFileMethods::GetCreateFile(), #CheckBox_CreateEncoded)
    MethodGadgetsStatus(EncodedFileMethods::GetOpenFile(), #CheckBox_OpenEncoded)
    MethodGadgetsStatus(EncodedFileMethods::GetReadFile(), #CheckBox_ReadEncoded)
    SelectComboBoxGadgetItem(#ComboBox_EncodedFile_Module, EncodedFileMethods::GetExternalModuleToUse())
   
    MethodGadgetsStatus(CryptedFileMethods::GetReadWrite(), #CheckBox_ReadWriteCrypted)
    MethodGadgetsStatus(CryptedFileMethods::GetCreateFile(), #CheckBox_CreateCrypted)
    MethodGadgetsStatus(CryptedFileMethods::GetOpenFile(), #CheckBox_OpenCrypted)
    MethodGadgetsStatus(CryptedFileMethods::GetReadFile(), #CheckBox_ReadCrypted)
    SelectComboBoxGadgetItem(#ComboBox_CryptedFile_Module, CryptedFileMethods::GetExternalModuleToUse())
    
    MethodGadgetsStatus(DebuggingMethods::GetDebugging(), #CheckBox_Debugging)
    
    Field::Reset(Field::#RESET_STYLE_AFTER_RUN)
   
  EndProcedure
  
  Procedure SaveSourceCode(FileName.s)
    
    FileID = CreateFile(#PB_Any, FileName)
    
    If IsFile(FileID)
      
      WriteStringFormat(FileID, #PB_UTF8)
      WriteString(FileID, GoScintilla::GetText(#Editor_Source_Code))
      CloseFile(FileID)
      
      SetFileDate(Filename, #PB_Date_Created, Date())
      SetFileDate(Filename, #PB_Date_Modified, Date())
      SetFileDate(Filename, #PB_Date_Accessed, Date())
      
    EndIf
    
  EndProcedure
  
  Procedure Refresh()
    
    Field::Reset(Field::#RESET_STYLE_NEW_MODULE)
    
    BasicMethods::Reset()
    StaticArrayMethods::Reset()
    LinkedListMethods::Reset()
    DynamicArrayMethods::Reset()
    MathsMethods::Reset()
    XMLFileMethods::Reset()
    JSONFileMethods::Reset()
    DataBaseMethods::Reset()
    PrefsFileMethods::Reset()
    BinaryFileMethods::Reset()
    EncodedFileMethods::Reset()
    CryptedFileMethods::Reset()
    DebuggingMethods::Reset()
    
    SetGadgetState(#Panel, 0)
    SetGadgetState(#Panel_Primary_Secondary, 0)
    
    To_GUI()
    
  EndProcedure
 
  Procedure SaveDevModuleProjectFile()
    
    FileName.s = SaveFileRequester(Language::Message("Select project to save"), Options::GetProjectSavePath(), Language::Message("DevModule Project (*.devm)|*.devm"), 1)
    
    If FileName <> ""
      
      From_GUI()
      
      FileName = CheckFileExtension::DoIt(FileName, "devm")
      
      If FileSize(FileName) <> -1
        
        If MessageRequester(Language::Message("Warning - File already exist"), ReplaceString(Language::Message("The file %FILE% already exist. Would you like to overwrite ?"), "%FILE%", Chr(34) + GetFilePart(FileName) + Chr(34)), #PB_MessageRequester_YesNo) = #PB_MessageRequester_Yes
          
          If DeleteFile(FileName)
            CodeGenerator::SaveProjectFile(FileName)
          EndIf
          
        EndIf
        
      Else
        
        CodeGenerator::SaveProjectFile(FileName)
        
      EndIf
      
    EndIf
    
  EndProcedure
  
  Procedure SaveDevModuleSourceCode()
    
    FileName.s = SaveFileRequester(Language::Message("Select the Source code to save"), Options::GetProjectSavePath(), Language::Message("PureBasic SourceCode (*.pb)|*.pb"), 1)
    
    If FileName <> ""
      
      FileName = CheckFileExtension::DoIt(FileName, "pb")
      
      If FileSize(FileName) <> -1
        
        If MessageRequester(Language::Message("Warning - File already exist"), ReplaceString(Language::Message("The file %FILE% already exist. Would you like to overwrite ?"), "%FILE%", Chr(34) + GetFilePart(FileName) + Chr(34)), #PB_MessageRequester_YesNo) = #PB_MessageRequester_Yes
          
          If DeleteFile(FileName)
            SaveSourceCode(FileName)
          EndIf
          
        EndIf
        
      Else
        
        SaveSourceCode(FileName)
        
      EndIf
      
    EndIf
    
  EndProcedure
  
  Procedure LoadDevModuleProjectFile()
    
    FileName.s = OpenFileRequester(Language::Message("Select project to open"), Options::GetProjectSavePath(), Language::Message("DevModule Project (*.devm)|*.devm"), 1)
    
    If FileName <> ""
      
      Field::Reset(Field::#RESET_STYLE_NEW_MODULE)
      
      CodeGenerator::LoadProjectFile(FileName)
      To_GUI()
      
      If GetGadgetState(#Option_Single_Instance) = 1
        DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_CountRowInTable, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
        DependentGadgets::DoIt(1, #CheckBox_DataBase_OpenFile, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable) 
      EndIf
      
      If GetGadgetState(#Option_Multiple_Instance_ID) = 1
        DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
      EndIf
      
      If GetGadgetState(#Option_Multiple_Instance_Index) = 1
        DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
      EndIf
      
      If GetGadgetState(#Option_Multiple_Instance_Name) = 1
        DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
      EndIf

      If GetGadgetState(#Option_Off_Module_Instance) = 1
        DependentGadgets::DoIt(1, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)    
      EndIf
      
      SetGadgetState(#Panel, 0)
      SetGadgetState(#Panel_Primary_Secondary, 0)
      
    EndIf
    
  EndProcedure
  
  Procedure SaveFileDispatcher()

    Select GetGadgetState(#Panel)
    
    	Case 0, 1, 2, 3, 4
    	  SaveDevModuleProjectFile()
    	  
    	Case 5
    	  SaveDevModuleSourceCode()
    	  
    EndSelect
    
  EndProcedure
  
  Procedure MatrixGenerator()
    
    If MatrixGenerator::Runner(#MainWin)
      
      SetGadgetState(#Option_Off_Module_Instance, 1)
      DependentGadgets::DoIt(1, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
      
      For Index = #CheckBox_Equal To #CheckBox_Swapping Step 3
        If Index <> #CheckBox_Divide
          SetGadgetState(Index, 1)
          DisableGadget(Index + 1, 0)
          DisableGadget(Index + 2, 0)
        EndIf
      Next
      
    EndIf
    
  EndProcedure
  
  Procedure MatrixEncodedGenerator()
    
    If MatrixEncodedGenerator::Runner(#MainWin)
      GoScintilla::SetText(#Editor_Source_Code, CodeGenerator::Generate(1))
      SetGadgetState(#Panel, 5)
    EndIf
    
  EndProcedure
  
  Procedure.i MyLineStyler(GadgetID, *utf8Buffer.ASCII, numUtf8Bytes, currentLine, startLine, originalEndLine)
    
    Protected result = GoScintilla::#STYLE_LINES_AS_REQUIRED, numBytesStyled, symbolJustStyled$, *ptrAscii.ASCII
    Static regexHEX
    Static regexBIN
    Static regexFunc
    Static regexFuncName
    Static regexType
    Static regexModuleType
    Static regexModule
    Static regexPointer
    
    If Not regexHEX
      ; Init search patterns
      #Highlight_Remove = -1
      #Highlight_None = 0
      #Highlight_Visible = 1
      #Highlight_End = 2
      
      regexHEX = CreateRegularExpression(#PB_Any, "^[$][0-9A-Fa-f]+")
      regexBIN = CreateRegularExpression(#PB_Any, "^[%][0-1]+")
      regexFunc = CreateRegularExpression(#PB_Any, "^^[\w]+[$]?([.][\w]+)?(::)?([\w]+)?[(]")
      regexFuncName = CreateRegularExpression(#PB_Any, "^[\w]+[$]?")
      regexType = CreateRegularExpression(#PB_Any, "^[.][a-zA-Z_][\w]*")
      regexModuleType = CreateRegularExpression(#PB_Any, "^[.]([a-zA-Z_][\w]*)(::)([a-zA-Z_][\w]*)")
      regexModule = CreateRegularExpression(#PB_Any, "^([a-zA-Z_][\w]*)(::)")
      regexPointer = CreateRegularExpression(#PB_Any, "^[\*][a-zA-Z_][\w]*")
      
    EndIf
    
    lineHighlighting = GoScintilla::GetLineData(GadgetID, currentLine-1)
    
    ;Need to loop through the UTF-8 buffer invoking GoScintilla's styling lexer as appropriate.
    While numUtf8Bytes
      Dim foundSymbol$(0)
      Select *utf8Buffer\a
        Case ';'
          ; search fold markers to update fold levels
          textToExamine$=LCase(PeekS(*utf8Buffer, numUtf8Bytes, #PB_UTF8))
          Select Left(textToExamine$, 2)
            Case ";{"
              GoScintilla::IncFoldLevel(GadgetID)
            Case ";}"
              GoScintilla::DecFoldLevel(GadgetID)
          EndSelect
          
        Case '.'
          ; search type
          textToExamine$ = LCase(PeekS(*utf8Buffer, numUtf8Bytes, #PB_UTF8))
          
          If ExtractRegularExpression(regexModuleType, textToExamine$, foundSymbol$())
            
            If ExamineRegularExpression(regexModuleType, foundSymbol$(0))
              ;apply separator style
              numBytesStyled = GoScintilla::StyleNextSymbol(GadgetID, *utf8Buffer, numUtf8Bytes)
              numUtf8Bytes-numBytesStyled
              *utf8Buffer+numBytesStyled
              
              ;apply Module style
              NextRegularExpressionMatch(regexModuleType)
              numBytesStyled = RegularExpressionGroupLength(regexModuleType, 1) + 1 - numBytesStyled
              ScintillaSendMessage(GadgetID, #SCI_SETSTYLING, numBytesStyled, #STYLES_MODULES)
              numUtf8Bytes-numBytesStyled
              *utf8Buffer+numBytesStyled
              
              ;apply separator style
              numBytesStyled = GoScintilla::StyleNextSymbol(GadgetID, *utf8Buffer, numUtf8Bytes)
              numUtf8Bytes-numBytesStyled
              *utf8Buffer+numBytesStyled
              
              ;apply separator style
              numBytesStyled = GoScintilla::StyleNextSymbol(GadgetID, *utf8Buffer, numUtf8Bytes)
              numUtf8Bytes-numBytesStyled
              *utf8Buffer+numBytesStyled
              
              numBytesStyled = RegularExpressionGroupLength(regexModuleType, 3) + 1 - numBytesStyled
              ScintillaSendMessage(GadgetID, #SCI_SETSTYLING, numBytesStyled, #STYLES_TYPES)
              numUtf8Bytes-numBytesStyled
              *utf8Buffer+numBytesStyled
              Continue
              
            EndIf
            
          ElseIf ExtractRegularExpression(regexType, textToExamine$, foundSymbol$())
            ;apply separator style
            numBytesStyled = GoScintilla::StyleNextSymbol(GadgetID, *utf8Buffer, numUtf8Bytes)
            numUtf8Bytes-numBytesStyled
            *utf8Buffer+numBytesStyled
            ;apply type style
            numBytesStyled=Len(foundSymbol$(0))-numBytesStyled
            Select LCase(foundSymbol$(0))
              Case ".a", ".b", ".c", ".d", ".f", ".i", ".l", ".q", ".s", ".u", ".w"
                ScintillaSendMessage(GadgetID, #SCI_SETSTYLING, numBytesStyled, 0)
              Default
                ScintillaSendMessage(GadgetID, #SCI_SETSTYLING, numBytesStyled, #STYLES_TYPES)
            EndSelect
            numUtf8Bytes-numBytesStyled
            *utf8Buffer+numBytesStyled
            Continue
          EndIf
         
        Case '$', '%'
          ; search BIN or HEX number
          textToExamine$ = LCase(PeekS(*utf8Buffer, numUtf8Bytes, #PB_UTF8))
          If ExtractRegularExpression(regexHEX, textToExamine$, foundSymbol$()) Or ExtractRegularExpression(regexBIN, textToExamine$, foundSymbol$())
            ;apply number style
            numBytesStyled = Len(foundSymbol$(0))
            ScintillaSendMessage(GadgetID, #SCI_SETSTYLING, numBytesStyled, #STYLES_NUMBERS)
            numUtf8Bytes-numBytesStyled
            *utf8Buffer+numBytesStyled
            Continue
          EndIf
          
          
        Case '_', 'a' To 'z', 'A' To 'Z'
          ; search function name
          textToExamine$ = LCase(PeekS(*utf8Buffer, numUtf8Bytes, #PB_UTF8))
          
          If MatchRegularExpression(regexModule, textToExamine$)
            If ExtractRegularExpression(regexModule, textToExamine$, foundSymbol$())
              ; apply Module style
              numBytesStyled = Len(foundSymbol$(0))-2
              ScintillaSendMessage(GadgetID, #SCI_SETSTYLING, numBytesStyled, #STYLES_MODULES)
              numUtf8Bytes-numBytesStyled
              *utf8Buffer+numBytesStyled
              
              ;apply separator style
              numBytesStyled = GoScintilla::StyleNextSymbol(GadgetID, *utf8Buffer, numUtf8Bytes)
              numUtf8Bytes-numBytesStyled
              *utf8Buffer+numBytesStyled
              
              ;apply separator style
              numBytesStyled = GoScintilla::StyleNextSymbol(GadgetID, *utf8Buffer, numUtf8Bytes)
              numUtf8Bytes-numBytesStyled
              *utf8Buffer+numBytesStyled
              Continue
            EndIf
          EndIf
          
          If MatchRegularExpression(regexFunc, textToExamine$)
            If ExtractRegularExpression(regexFuncName, textToExamine$, foundSymbol$())
              ;apply function style
              numBytesStyled = Len(foundSymbol$(0))
              ScintillaSendMessage(GadgetID, #SCI_SETSTYLING, numBytesStyled, #STYLES_FUNCTIONS)
              numUtf8Bytes-numBytesStyled
              *utf8Buffer+numBytesStyled
              Continue
            EndIf
          EndIf
          
        Case '*'
          ; search pointer name
          textToExamine$ = LCase(PeekS(*utf8Buffer, numUtf8Bytes, #PB_UTF8))
          If MatchRegularExpression(regexPointer, textToExamine$)
            If ExtractRegularExpression(regexPointer, textToExamine$, foundSymbol$())
              ;apply function style
              numBytesStyled = Len(foundSymbol$(0))
              ScintillaSendMessage(GadgetID, #SCI_SETSTYLING, numBytesStyled, #STYLES_POINTERS)
              numUtf8Bytes-numBytesStyled
              *utf8Buffer+numBytesStyled
              Continue
            EndIf
          EndIf
          
      EndSelect
      
      numBytesStyled = GoScintilla::StyleNextSymbol(GadgetID, *utf8Buffer, numUtf8Bytes)
      symbolJustStyled$ = LCase(PeekS(*utf8Buffer, numBytesStyled, #PB_UTF8))
      
      If symbolJustStyled$="procedure"
        lineHighlighting=#Highlight_Visible
      ElseIf symbolJustStyled$="endprocedure"
        If lineHighlighting=#Highlight_Visible
          lineHighlighting=#Highlight_End
        Else
          lineHighlighting=#Highlight_Remove
        EndIf
      EndIf
      
      numUtf8Bytes-numBytesStyled
      *utf8Buffer+numBytesStyled
      
    Wend
    
    If GoScintilla::GetLineData(GadgetID, currentLine) <> lineHighlighting
      Select lineHighlighting
        Case #Highlight_Visible
          GoScintilla::SetLineData(GadgetID, currentLine, #Highlight_Visible)
          GoScintilla::SetLineHighlighted(GadgetID,currentLine, 1)
        Case #Highlight_End
          GoScintilla::SetLineData(GadgetID, currentLine, #Highlight_None)
          GoScintilla::SetLineHighlighted(GadgetID, currentLine, 1)
        Case #Highlight_None, #Highlight_Remove
          GoScintilla::SetLineData(GadgetID, currentLine, #Highlight_None)
          GoScintilla::SetLineHighlighted(GadgetID, currentLine, 0)
      EndSelect
      result = GoScintilla::#STYLE_NEXT_LINE_REGARDLESS
    EndIf
    
    ProcedureReturn result
  EndProcedure
  
  Procedure GoScintillaSetup(EditorID, FullSetup.b = #True)
    
    ;Set the padding added to the width of the line-number margin.
    GoScintilla::SetAttribute(EditorID,  GoScintilla::#LINE_NUMBER_AUTOSIZE_PADDING, 10)
    GoScintilla::SetMarginWidth(EditorID,  GoScintilla::#MARGIN_FOLDING_SYMBOLS, 14)
    GoScintilla::SetFont(EditorID, Options::GetFontName(), Options::GetFontSize())
    
    If Comments::GetTabOption() = 0
      GoScintilla::SetTabs(EditorID, Comments::GetTabSize(), #True)
    Else
      GoScintilla::SetTabs(EditorID, 1, #False)
    EndIf
    
    ;Set colors of text area
    GoScintilla::SetColor(EditorID, GoScintilla::#FORE_COLOR, SyntaxHighlighting::GetDefaultForeColor())
    GoScintilla::SetColor(EditorID, GoScintilla::#BACK_COLOR, SyntaxHighlighting::GetDefaultBackColor())
    
    ;Set colors of caret (caret color and the back color of the line containing the caret)
    GoScintilla::SetColor(EditorID, GoScintilla::#CARET_FORE_COLOR, SyntaxHighlighting::GetCaretForeColor())
    GoScintilla::SetColor(EditorID, GoScintilla::#CARET_LINE_BACK_COLOR, SyntaxHighlighting::GetCaretBackColor())
    
    ;Set colors of selection
    GoScintilla::SetColor(EditorID, GoScintilla::#SELECTION_BACK_COLOR, SyntaxHighlighting::GetSelectionBackColor())
    GoScintilla::SetColor(EditorID, GoScintilla::#SELECTION_FORE_COLOR, SyntaxHighlighting::GetSelectionForeColor())
    
    ;Set colors of line margin
    GoScintilla::SetColor(EditorID, GoScintilla::#LINE_NUMBER_FORE_COLOR, SyntaxHighlighting::GetLineNumbersForeColor())
    GoScintilla::SetColor(EditorID, GoScintilla::#LINE_NUMBER_BACK_COLOR, SyntaxHighlighting::GetLineNumbersBackColor())
    
    ;Set colors of fold margin
    GoScintilla::SetColor(EditorID, GoScintilla::#FOLD_MARGIN_HI_BACK_COLOR, SyntaxHighlighting::GetFoldMarginHiBackColor())
    GoScintilla::SetColor(EditorID, GoScintilla::#FOLD_MARGIN_LO_BACK_COLOR, SyntaxHighlighting::GetFoldMarginLoBackColor())
    
    ;Set colors of fold markers
    GoScintilla::SetColor(EditorID, GoScintilla::#FOLD_MARKERS_FORE_COLOR, SyntaxHighlighting::GetFoldMarkerForeColor())
    GoScintilla::SetColor(EditorID, GoScintilla::#FOLD_MARKERS_BACK_COLOR, SyntaxHighlighting::GetFoldMarkerBackColor())
    
    GoScintilla::SetLineHighlightColor(EditorID, SyntaxHighlighting::GetLineHighlightColor())
    
    If SyntaxHighlighting::GetKeywordsInBold() = #True
      GoScintilla::SetStyleFont(EditorID, #STYLES_COMMANDS, "", -1, #PB_Font_Bold)
    EndIf
    
    GoScintilla::SetStyleColors(EditorID, #STYLES_COMMANDS, SyntaxHighlighting::GetKeywordsColor())
    GoScintilla::SetStyleColors(EditorID, #STYLES_COMMENTS, SyntaxHighlighting::GetCommentColor()) 
    GoScintilla::SetStyleColors(EditorID, #STYLES_LITERAL_STRINGS, SyntaxHighlighting::GetLiteralStringColor()) 
    GoScintilla::SetStyleColors(EditorID, #STYLES_NUMBERS, SyntaxHighlighting::GetNumbersColor())
    GoScintilla::SetStyleColors(EditorID, #STYLES_CONSTANTS, SyntaxHighlighting::GetConstantsColor()) 
    GoScintilla::SetStyleColors(EditorID, #STYLES_FUNCTIONS, SyntaxHighlighting::GetProceduresColor())
    GoScintilla::SetStyleColors(EditorID, #STYLES_SEPARATORS, SyntaxHighlighting::GetSeparatorsColor())
    GoScintilla::SetStyleColors(EditorID, #STYLES_OPERATORS, SyntaxHighlighting::GetOperatorsColor())
    GoScintilla::SetStyleColors(EditorID, #STYLES_TYPES, SyntaxHighlighting::GetTypesColor())
    GoScintilla::SetStyleColors(EditorID, #STYLES_MODULES, SyntaxHighlighting::GetModulesColor())
    GoScintilla::SetStyleColors(EditorID, #STYLES_POINTERS, SyntaxHighlighting::GetPointersColor())
    
    If FullSetup.b = #True
      
      ;Set delimiters and keywords for our syntax highlighting.
      ;========================================================
      ;Delimiters.
      ;First set up a ; symbol to denote a comment.
      GoScintilla::AddDelimiter(EditorID, ";", "", GoScintilla::#DELIMIT_TO_END_OF_LINE, #STYLES_COMMENTS)
      ;Now set up quotes to denote literal strings.
      GoScintilla::AddDelimiter(EditorID, Chr(34), Chr(34), GoScintilla::#DELIMIT_BETWEEN, #STYLES_LITERAL_STRINGS)
      ;Now set up a # symbol to denote a constant.
      GoScintilla::AddDelimiter(EditorID, "#", "", GoScintilla::#LEFT_DELIMIT_WITHOUT_WHITESPACE, #STYLES_CONSTANTS)
      
      ;Basic command keywords.
      GoScintilla::AddKeywords(EditorID, SyntaxHighlighting::BuiltKeywordList(), #STYLES_COMMANDS)
      
      ;Add some folding keywords.
      GoScintilla::AddKeywords(EditorID, SyntaxHighlighting::BuiltOpenFoldKeywordList(), #STYLES_COMMANDS, GoScintilla::#OPEN_FOLD_KEYWORD)
      GoScintilla::AddKeywords(EditorID, SyntaxHighlighting::BuiltCloseFoldKeywordList(), #STYLES_COMMANDS, GoScintilla::#CLOSE_FOLD_KEYWORD)
      
      ;Add separator keywords
      GoScintilla::AddKeywords(EditorID, SyntaxHighlighting::GetSeparators(), #STYLES_SEPARATORS)
      
      ;Add operator keywords
      GoScintilla::AddKeywords(EditorID, SyntaxHighlighting::GetOperators(), #STYLES_OPERATORS)
      
      ;Additional lexer options.
      ;=========================
      ;The lexer needs to know what separator characters we are using.
      GoScintilla::SetLexerOption(EditorID,  GoScintilla::#LEXER_OPTION_SEPARATOR_SYMBOLS, @"=+-*/%&|<>(){}[],.:\")
      ;We can also set a style for numbers.
      GoScintilla::SetLexerOption(EditorID,  GoScintilla::#LEXER_OPTION_NUMBERS_STYLE_INDEX, #STYLES_NUMBERS)
      
      ;Set our user-defined line styling function.
      ;===========================================
      GoScintilla::SetLineStylingFunction(EditorID,  @MyLineStyler())
      
    EndIf
    
  EndProcedure
  
  Procedure Private_ScanInternalStructure(List FieldType.s(), EditorID)
    
    Code.s = GoScintilla::GetText(EditorID)
    LineMax = CountString(Code, LineFeed::Single()) + 1
    
    For LineID = 1 To LineMax
      
      Field.s = StringField(Code, LineID, LineFeed::Single())
      
      If FindString(Field, "::")
        FieldType.s = ""
      ElseIf FindString(Field, "(")
        FieldType = StringBetween::FindIt(Field, ".", "(")
      ElseIf FindString(Field, "[")
        FieldType = StringBetween::FindIt(Field, ".", "[")
      Else
        FieldType = StringField(Field, 2, ".")
      EndIf
      
      If Len(FieldType) > 1
        
        FoundAt.l = -1
        
        ForEach FieldType()
          If FieldType() = FieldType
            FoundAt = ListIndex(FieldType())
            Break
          EndIf
        Next
        
        If FoundAt = -1
          AddElement(FieldType())
          FieldType() = FieldType
        EndIf
        
      EndIf
      
    Next
    
  EndProcedure
  
  Procedure ScanInternalStructure()
    
    ClearGadgetItems(#ListView_Internal_Structure)
    Field::Reset(Field::#RESET_STYLE_FLUSH_INTERNAL_STRUCTURE)
    
    NewList FieldType.s()
    
    Private_ScanInternalStructure(FieldType(), #Editor_Module_Primary)
    Private_ScanInternalStructure(FieldType(), #Editor_Module_Secondary)
    
    ForEach FieldType()
      
      AddGadgetItem(#ListView_Internal_Structure, -1, FieldType()) 
      
      If General::LinearSearchStructByName(FieldType()) = -1
        General::AddStruct()
        General::SetStructName(FieldType())
      EndIf
      
      FieldType() = ""
      
    Next
    
    FreeList(FieldType())
    
  EndProcedure
  
  Procedure InternalStructureEditor()
    
    StructName.s = GetGadgetText(#ListView_Internal_Structure)
    
    If General::LinearSearchStructByName(StructName) >= 0
      
      If Window::OpenEx(#InternalStructureWin, Language::Message("Internal Structure Editor"))
        
        TextGadget(#Text_Internal_Structure_Name, 5,5, Window::Width(#InternalStructureWin)-10, 25, Language::Message("Structure name") + " : " + StructName)
        GoScintilla::Gadget(#Editor_Internal_Structure, 5, GadgetLocate::DownOf(#Text_Internal_Structure_Name, 5), Window::Width(#InternalStructureWin)-10, 325, 0, GoScintilla::#AUTOSIZE_LINE_NUMBERS_MARGIN | GoScintilla::#KEYWORDS_CASE_SENSITIVE)
        GoScintillaSetup(#Editor_Internal_Structure)
        
        General::To_GUI(1)
        
        ButtonGroup::Horizontal(#Button_Internal_Structure_OK, #Button_Internal_Structure_CANCEL, 500-150*2-10, 365, 150, 30, 5)
        
        SetGadgetText(#Button_Internal_Structure_OK, Language::Message("OK"))
        SetGadgetText(#Button_Internal_Structure_CANCEL, Language::Message("Cancel"))
        
      EndIf
      
    EndIf
    
  EndProcedure
  
  Procedure CloseInternalStructureEditor(Mode.b)
    
    If Mode = 1
      PastedCodeCleanUp(#Editor_Internal_Structure)
      General::From_GUI(1)
    EndIf
    
    GoScintilla::Free(#Editor_Internal_Structure)
    Window::GetCurrentPositionSize(#InternalStructureWin)
    CloseWindow(#InternalStructureWin)
    HideWindow(#MainWin, #False)
    
  EndProcedure
  
  Procedure OptionsEditor()
    
    If Window::OpenEx(#OptionsWin, Language::Message("Options"))
      
      HideWindow(#MainWin, #True)
      
      CompilerSelect #PB_Compiler_OS
          
        CompilerCase #PB_OS_Windows 
          
          BrowsePathGadgetGroup(#Frame_Project_SavePath, 5, 5, 650)
          MethodsGadgetTextStringGroup(5, GadgetLocate::DownOf(#Frame_Project_SavePath, 5), #Frame_User, #Text_User_Email, 320)
          
          FrameGadget(#Frame_Source_Formatting, 5, GadgetLocate::DownOf(#Frame_User, 5), 320, 115, Language::Message("Source code formatting"))
          OptionGadget(#Option_Use_NBSPACE_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetY(#Frame_Source_Formatting) + 20, 295, 20, Language::Message("Tabulation (Number of space)"))
          OptionGadget(#Option_Use_ASCII_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 40), 295, 20, Language::Message("Use the real tabulation character"))
          TrackBarGadgetGroup(#TrackBar_Tabulation_Size, #Text_Tabulation_Size, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 5), 305)
          
          DateFormatGadgetGroup(#Frame_Date_Formatting, 5, GadgetLocate::DownOf(#Frame_Source_Formatting, 5), 320)
          
          FrameGadget(#Frame_Editor_Color, 5, GadgetLocate::DownOf(#Frame_Date_Formatting, 5), 320, 100, Language::Message("Editors Font"))
          TextGadget(#Text_Editor_FontName, 10, GadgetY(#Frame_Editor_Color) + 20, 125, 34, Language::Message("Font"), #PB_Text_Center)
          StringGadget(#String_Editor_FontName, GadgetLocate::RightOf(#Text_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 125, 34, "", #PB_String_ReadOnly)
          ButtonGadget(#Button_Editor_FontSelect, GadgetLocate::RightOf(#String_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 50, 34, "...")
          TextGadget(#Text_Editor_FontSize, 10, GadgetLocate::DownOf(#Text_Editor_FontName, 5), 125, 34, Language::Message("Size"), #PB_Text_Center)
          StringGadget(#String_Editor_FontSize, GadgetLocate::RightOf(#Text_Editor_FontSize, 5), GadgetY(#Text_Editor_FontSize), 125, 34, "",  #PB_String_ReadOnly)
          
          FrameGadget(#Frame_Keyword_Bold, 5, GadgetLocate::DownOf(#Frame_Editor_Color, 5), 320, 55, Language::Message("Keywords"))
          CheckBoxGadget(#CheckBox_Keyword_Bold, 10, GadgetY(#Frame_Keyword_Bold) + 20, 300, 34, Language::Message("Keywords in bold"))
          
          FrameGadget(#Frame_Syntax_Colors, GadgetLocate::RightOf(#Frame_Date_Formatting, 5), GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 325, 490, Language::Message("Syntax coloring"))
          ScrollAreaGadget(#Scroll_Syntax_Colors, GadgetLocate::RightOf(#Frame_Date_Formatting, 10), GadgetY(#Frame_Syntax_Colors) + 20, 315, 465, 290, 940)
            
            StringGadget(#Text_DefaultForeColor, 5,5, 225,34, Language::Message("Normal Text"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_DefaultForeColor, 235, 5, 50, 32, SyntaxHighlighting::GetDefaultForeColor())
            
            StringGadget(#Text_DefaultBackColor, 5, GadgetLocate::DownOf(#Text_DefaultForeColor, 5), 225, 34, Language::Message("Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_DefaultBackColor, 235, GadgetY(#Text_DefaultBackColor), 50, 32, SyntaxHighlighting::GetDefaultBackColor())
            
            StringGadget(#Text_CaretForeColor, 5, GadgetLocate::DownOf(#Text_DefaultBackColor, 5), 225, 34, Language::Message("Caret"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_CaretForeColor, 235, GadgetY(#Text_CaretForeColor), 50, 32, SyntaxHighlighting::GetCaretForeColor())
            
            StringGadget(#Text_CaretBackColor, 5, GadgetLocate::DownOf(#Text_CaretForeColor, 5), 225, 34, Language::Message("Current line Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_CaretBackColor, 235, GadgetY(#Text_CaretBackColor), 50, 32, SyntaxHighlighting::GetCaretBackColor())
            
            StringGadget(#Text_SelectionForeColor, 5, GadgetLocate::DownOf(#Text_CaretBackColor, 5), 225, 34, Language::Message("Selection"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_SelectionForeColor, 235, GadgetY(#Text_SelectionForeColor), 50, 32, SyntaxHighlighting::GetSelectionForeColor())
            
            StringGadget(#Text_SelectionBackColor, 5, GadgetLocate::DownOf(#Text_SelectionForeColor, 5), 225, 34, Language::Message("Selection Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_SelectionBackColor, 235, GadgetY(#Text_SelectionBackColor), 50, 32, SyntaxHighlighting::GetSelectionBackColor())
            
            StringGadget(#Text_LineNumbersForeColor, 5, GadgetLocate::DownOf(#Text_SelectionBackColor, 5), 225, 34, Language::Message("Line Numbers"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_LineNumbersForeColor, 235, GadgetY(#Text_LineNumbersForeColor), 50, 32, SyntaxHighlighting::GetLineNumbersForeColor())
            
            StringGadget(#Text_LineNumbersBackColor, 5, GadgetLocate::DownOf(#Text_LineNumbersForeColor, 5), 225, 34, Language::Message("Line Numbers Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_LineNumbersBackColor, 235, GadgetY(#Text_LineNumbersBackColor), 50, 32, SyntaxHighlighting::GetLineNumbersBackColor())
            
            StringGadget(#Text_FoldMarginHiBackColor, 5, GadgetLocate::DownOf(#Text_LineNumbersBackColor, 5), 225, 34, Language::Message("Fold Margin Background") + " (Hi)",  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_FoldMarginHiBackColor, 235, GadgetY(#Text_FoldMarginHiBackColor), 50, 32, SyntaxHighlighting::GetFoldMarginHiBackColor())
            
            StringGadget(#Text_FoldMarginLoBackColor, 5, GadgetLocate::DownOf(#Text_FoldMarginHiBackColor, 5), 225, 34, Language::Message("Fold Margin Background") + " (Lo)",  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_FoldMarginLoBackColor, 235, GadgetY(#Text_FoldMarginLoBackColor), 50, 32, SyntaxHighlighting::GetFoldMarginLoBackColor())
            
            StringGadget(#Text_FoldMarkerForeColor, 5, GadgetLocate::DownOf(#Text_FoldMarginLoBackColor, 5), 225, 34, Language::Message("Fold Markers"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_FoldMarkerForeColor, 235, GadgetY(#Text_FoldMarkerForeColor), 50, 32, SyntaxHighlighting::GetFoldMarkerForeColor())
            
            StringGadget(#Text_FoldMarkerBackColor, 5, GadgetLocate::DownOf(#Text_FoldMarkerForeColor, 5), 225, 34, Language::Message("Fold Markers Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_FoldMarkerBackColor, 235, GadgetY(#Text_FoldMarkerBackColor), 50, 32, SyntaxHighlighting::GetFoldMarkerBackColor())
            
            StringGadget(#Text_LineHighlightColor, 5, GadgetLocate::DownOf(#Text_FoldMarkerBackColor, 5), 225, 34, Language::Message("Line Highlight"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_LineHighlightColor, 235, GadgetY(#Text_LineHighlightColor), 50, 32, SyntaxHighlighting::GetLineHighlightColor())
            
            StringGadget(#Text_KeywordsColor, 5, GadgetLocate::DownOf(#Text_LineHighlightColor, 5), 225, 34, Language::Message("Keywords"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_KeywordsColor, 235, GadgetY(#Text_KeywordsColor), 50, 32, SyntaxHighlighting::GetKeywordsColor())
            
            StringGadget(#Text_CommentColor, 5, GadgetLocate::DownOf(#Text_KeywordsColor, 5), 225, 34, Language::Message("Comments"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_CommentColor, 235, GadgetY(#Text_CommentColor), 50, 32, SyntaxHighlighting::GetCommentColor())
            
            StringGadget(#Text_LiteralStringColor, 5, GadgetLocate::DownOf(#Text_CommentColor, 5), 225, 34, Language::Message("Literal Strings"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_LiteralStringColor, 235, GadgetY(#Text_LiteralStringColor), 50, 32, SyntaxHighlighting::GetLiteralStringColor())
            
            StringGadget(#Text_NumbersColor, 5, GadgetLocate::DownOf(#Text_LiteralStringColor, 5), 225, 34, Language::Message("Numbers"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_NumbersColor, 235, GadgetY(#Text_NumbersColor), 50, 32, SyntaxHighlighting::GetNumbersColor())
            
            StringGadget(#Text_ConstantsColor, 5, GadgetLocate::DownOf(#Text_NumbersColor, 5), 225, 34, Language::Message("Constants"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_ConstantsColor, 235, GadgetY(#Text_ConstantsColor), 50, 32, SyntaxHighlighting::GetConstantsColor())
            
            StringGadget(#Text_ProceduresColor, 5, GadgetLocate::DownOf(#Text_ConstantsColor, 5), 225, 34, Language::Message("Procedures"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_ProceduresColor, 235, GadgetY(#Text_ProceduresColor), 50, 32, SyntaxHighlighting::GetProceduresColor())
            
            StringGadget(#Text_SeperatorsColor, 5, GadgetLocate::DownOf(#Text_ProceduresColor, 5), 225, 34, Language::Message("Separators"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_SeperatorsColor, 235, GadgetY(#Text_SeperatorsColor), 50, 32, SyntaxHighlighting::GetSeparatorsColor())
            
            StringGadget(#Text_OperatorsColor, 5, GadgetLocate::DownOf(#Text_SeperatorsColor, 5), 225, 34, Language::Message("Operators"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_OperatorsColor, 235, GadgetY(#Text_OperatorsColor), 50, 32, SyntaxHighlighting::GetOperatorsColor())
            
            StringGadget(#Text_TypesColor, 5, GadgetLocate::DownOf(#Text_OperatorsColor, 5), 225, 34, Language::Message("Types"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_TypesColor, 235, GadgetY(#Text_TypesColor), 50, 32, SyntaxHighlighting::GetTypesColor())
            
            StringGadget(#Text_ModulesColor, 5, GadgetLocate::DownOf(#Text_TypesColor, 5), 225, 34, Language::Message("Modules"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_ModulesColor, 235, GadgetY(#Text_ModulesColor), 50, 32, SyntaxHighlighting::GetModulesColor())
            
            StringGadget(#Text_PointersColor, 5, GadgetLocate::DownOf(#Text_ModulesColor, 5), 225, 34, Language::Message("Pointers"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_PointersColor, 235, GadgetY(#Text_PointersColor), 50, 32, SyntaxHighlighting::GetPointersColor())
          CloseGadgetList() 
          
          ButtonGroup::Horizontal(#Button_Options_OK, #Button_Options_CANCEL, 660-150*2-10, 570, 150, 34, 5)
          
        CompilerCase #PB_OS_Linux  
          
          BrowsePathGadgetGroup(#Frame_Project_SavePath, 5, 5, 650)
          MethodsGadgetTextStringGroup(5, GadgetLocate::DownOf(#Frame_Project_SavePath, 5), #Frame_User, #Text_User_Email, 320)
          
          FrameGadget(#Frame_Source_Formatting, 5, GadgetLocate::DownOf(#Frame_User, 5), 320, 115, Language::Message("Source code formatting"))
          OptionGadget(#Option_Use_NBSPACE_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetY(#Frame_Source_Formatting) + 20, 295, 20, Language::Message("Tabulation (Number of space)"))
          OptionGadget(#Option_Use_ASCII_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 40), 295, 20, Language::Message("Use the real tabulation character"))
          TrackBarGadgetGroup(#TrackBar_Tabulation_Size, #Text_Tabulation_Size, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 5), 305)
          
          DateFormatGadgetGroup(#Frame_Date_Formatting, 5, GadgetLocate::DownOf(#Frame_Source_Formatting, 5), 320)
          
          FrameGadget(#Frame_Editor_Color, 5, GadgetLocate::DownOf(#Frame_Date_Formatting, 5), 320, 100, Language::Message("Editors Font"))
          TextGadget(#Text_Editor_FontName, 10, GadgetY(#Frame_Editor_Color) + 20, 125, 34, Language::Message("Font"), #PB_Text_Center)
          StringGadget(#String_Editor_FontName, GadgetLocate::RightOf(#Text_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 125, 34, "", #PB_String_ReadOnly)
          ButtonGadget(#Button_Editor_FontSelect, GadgetLocate::RightOf(#String_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 50, 34, "...")
          TextGadget(#Text_Editor_FontSize, 10, GadgetLocate::DownOf(#Text_Editor_FontName, 5), 125, 34, Language::Message("Size"), #PB_Text_Center)
          StringGadget(#String_Editor_FontSize, GadgetLocate::RightOf(#Text_Editor_FontSize, 5), GadgetY(#Text_Editor_FontSize), 125, 34, "",  #PB_String_ReadOnly)
          
          FrameGadget(#Frame_Keyword_Bold, 5, GadgetLocate::DownOf(#Frame_Editor_Color, 5), 320, 55, Language::Message("Keywords"))
          CheckBoxGadget(#CheckBox_Keyword_Bold, 10, GadgetY(#Frame_Keyword_Bold) + 20, 300, 34, Language::Message("Keywords in bold"))
          
          FrameGadget(#Frame_Syntax_Colors, GadgetLocate::RightOf(#Frame_Date_Formatting, 5), GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 325, 490, Language::Message("Syntax coloring"))
          ScrollAreaGadget(#Scroll_Syntax_Colors, GadgetLocate::RightOf(#Frame_Date_Formatting, 10), GadgetY(#Frame_Syntax_Colors) + 20, 315, 465, 315, 940)
            
            StringGadget(#Text_DefaultForeColor, 5,5, 250,34, Language::Message("Normal Text"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_DefaultForeColor, 260, 5, 50, 32, SyntaxHighlighting::GetDefaultForeColor())
            
            StringGadget(#Text_DefaultBackColor, 5, GadgetLocate::DownOf(#Text_DefaultForeColor, 5), 250, 34, Language::Message("Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_DefaultBackColor, 260, GadgetY(#Text_DefaultBackColor), 50, 32, SyntaxHighlighting::GetDefaultBackColor())
            
            StringGadget(#Text_CaretForeColor, 5, GadgetLocate::DownOf(#Text_DefaultBackColor, 5), 250, 34, Language::Message("Caret"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_CaretForeColor, 260, GadgetY(#Text_CaretForeColor), 50, 32, SyntaxHighlighting::GetCaretForeColor())
            
            StringGadget(#Text_CaretBackColor, 5, GadgetLocate::DownOf(#Text_CaretForeColor, 5), 250, 34, Language::Message("Current line Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_CaretBackColor, 260, GadgetY(#Text_CaretBackColor), 50, 32, SyntaxHighlighting::GetCaretBackColor())
            
            StringGadget(#Text_SelectionForeColor, 5, GadgetLocate::DownOf(#Text_CaretBackColor, 5), 250, 34, Language::Message("Selection"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_SelectionForeColor, 260, GadgetY(#Text_SelectionForeColor), 50, 32, SyntaxHighlighting::GetSelectionForeColor())
            
            StringGadget(#Text_SelectionBackColor, 5, GadgetLocate::DownOf(#Text_SelectionForeColor, 5), 250, 34, Language::Message("Selection Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_SelectionBackColor, 260, GadgetY(#Text_SelectionBackColor), 50, 32, SyntaxHighlighting::GetSelectionBackColor())
            
            StringGadget(#Text_LineNumbersForeColor, 5, GadgetLocate::DownOf(#Text_SelectionBackColor, 5), 250, 34, Language::Message("Line Numbers"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_LineNumbersForeColor, 260, GadgetY(#Text_LineNumbersForeColor), 50, 32, SyntaxHighlighting::GetLineNumbersForeColor())
            
            StringGadget(#Text_LineNumbersBackColor, 5, GadgetLocate::DownOf(#Text_LineNumbersForeColor, 5), 250, 34, Language::Message("Line Numbers Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_LineNumbersBackColor, 260, GadgetY(#Text_LineNumbersBackColor), 50, 32, SyntaxHighlighting::GetLineNumbersBackColor())
            
            StringGadget(#Text_FoldMarginHiBackColor, 5, GadgetLocate::DownOf(#Text_LineNumbersBackColor, 5), 250, 34, Language::Message("Fold Margin Background") + " (Hi)",  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_FoldMarginHiBackColor, 260, GadgetY(#Text_FoldMarginHiBackColor), 50, 32, SyntaxHighlighting::GetFoldMarginHiBackColor())
            
            StringGadget(#Text_FoldMarginLoBackColor, 5, GadgetLocate::DownOf(#Text_FoldMarginHiBackColor, 5), 250, 34, Language::Message("Fold Margin Background") + " (Lo)",  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_FoldMarginLoBackColor, 260, GadgetY(#Text_FoldMarginLoBackColor), 50, 32, SyntaxHighlighting::GetFoldMarginLoBackColor())
            
            StringGadget(#Text_FoldMarkerForeColor, 5, GadgetLocate::DownOf(#Text_FoldMarginLoBackColor, 5), 250, 34, Language::Message("Fold Markers"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_FoldMarkerForeColor, 260, GadgetY(#Text_FoldMarkerForeColor), 50, 32, SyntaxHighlighting::GetFoldMarkerForeColor())
            
            StringGadget(#Text_FoldMarkerBackColor, 5, GadgetLocate::DownOf(#Text_FoldMarkerForeColor, 5), 250, 34, Language::Message("Fold Markers Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_FoldMarkerBackColor, 260, GadgetY(#Text_FoldMarkerBackColor), 50, 32, SyntaxHighlighting::GetFoldMarkerBackColor())
            
            StringGadget(#Text_LineHighlightColor, 5, GadgetLocate::DownOf(#Text_FoldMarkerBackColor, 5), 250, 34, Language::Message("Line Highlight"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_LineHighlightColor, 260, GadgetY(#Text_LineHighlightColor), 50, 32, SyntaxHighlighting::GetLineHighlightColor())
            
            StringGadget(#Text_KeywordsColor, 5, GadgetLocate::DownOf(#Text_LineHighlightColor, 5), 250, 34, Language::Message("Keywords"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_KeywordsColor, 260, GadgetY(#Text_KeywordsColor), 50, 32, SyntaxHighlighting::GetKeywordsColor())
            
            StringGadget(#Text_CommentColor, 5, GadgetLocate::DownOf(#Text_KeywordsColor, 5), 250, 34, Language::Message("Comments"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_CommentColor, 260, GadgetY(#Text_CommentColor), 50, 32, SyntaxHighlighting::GetCommentColor())
            
            StringGadget(#Text_LiteralStringColor, 5, GadgetLocate::DownOf(#Text_CommentColor, 5), 250, 34, Language::Message("Literal Strings"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_LiteralStringColor, 260, GadgetY(#Text_LiteralStringColor), 50, 32, SyntaxHighlighting::GetLiteralStringColor())
            
            StringGadget(#Text_NumbersColor, 5, GadgetLocate::DownOf(#Text_LiteralStringColor, 5), 250, 34, Language::Message("Numbers"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_NumbersColor, 260, GadgetY(#Text_NumbersColor), 50, 32, SyntaxHighlighting::GetNumbersColor())
            
            StringGadget(#Text_ConstantsColor, 5, GadgetLocate::DownOf(#Text_NumbersColor, 5), 250, 34, Language::Message("Constants"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_ConstantsColor, 260, GadgetY(#Text_ConstantsColor), 50, 32, SyntaxHighlighting::GetConstantsColor())
            
            StringGadget(#Text_ProceduresColor, 5, GadgetLocate::DownOf(#Text_ConstantsColor, 5), 250, 34, Language::Message("Procedures"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_ProceduresColor, 260, GadgetY(#Text_ProceduresColor), 50, 32, SyntaxHighlighting::GetProceduresColor())
            
            StringGadget(#Text_SeperatorsColor, 5, GadgetLocate::DownOf(#Text_ProceduresColor, 5), 250, 34, Language::Message("Separators"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_SeperatorsColor, 260, GadgetY(#Text_SeperatorsColor), 50, 32, SyntaxHighlighting::GetSeparatorsColor())
            
            StringGadget(#Text_OperatorsColor, 5, GadgetLocate::DownOf(#Text_SeperatorsColor, 5), 250, 34, Language::Message("Operators"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_OperatorsColor, 260, GadgetY(#Text_OperatorsColor), 50, 32, SyntaxHighlighting::GetOperatorsColor())
            
            StringGadget(#Text_TypesColor, 5, GadgetLocate::DownOf(#Text_OperatorsColor, 5), 250, 34, Language::Message("Types"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_TypesColor, 260, GadgetY(#Text_TypesColor), 50, 32, SyntaxHighlighting::GetTypesColor())
            
            StringGadget(#Text_ModulesColor, 5, GadgetLocate::DownOf(#Text_TypesColor, 5), 250, 34, Language::Message("Modules"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_ModulesColor, 260, GadgetY(#Text_ModulesColor), 50, 32, SyntaxHighlighting::GetModulesColor())
            
            StringGadget(#Text_PointersColor, 5, GadgetLocate::DownOf(#Text_ModulesColor, 5), 250, 34, Language::Message("Pointers"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_PointersColor, 260, GadgetY(#Text_PointersColor), 50, 32, SyntaxHighlighting::GetPointersColor())
            
          CloseGadgetList() 
          
          ButtonGroup::Horizontal(#Button_Options_OK, #Button_Options_CANCEL, 660-150*2-10, 570, 150, 34, 5)
          
        CompilerCase #PB_OS_MacOS ; Maybe the interface need some adjustment
          
          BrowsePathGadgetGroup(#Frame_Project_SavePath, 5, 5, 650)
          MethodsGadgetTextStringGroup(5, GadgetLocate::DownOf(#Frame_Project_SavePath, 5), #Frame_User, #Text_User_Email, 320)
          
          FrameGadget(#Frame_Source_Formatting, 5, GadgetLocate::DownOf(#Frame_User, 5), 320, 115, Language::Message("Source code formatting"))
          OptionGadget(#Option_Use_NBSPACE_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetY(#Frame_Source_Formatting) + 20, 295, 20, Language::Message("Tabulation (Number of space)"))
          OptionGadget(#Option_Use_ASCII_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 40), 295, 20, Language::Message("Use the real tabulation character"))
          TrackBarGadgetGroup(#TrackBar_Tabulation_Size, #Text_Tabulation_Size, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 5), 305)
          
          DateFormatGadgetGroup(#Frame_Date_Formatting, 5, GadgetLocate::DownOf(#Frame_Source_Formatting, 5), 320)
          
          FrameGadget(#Frame_Editor_Color, 5, GadgetLocate::DownOf(#Frame_Date_Formatting, 5), 320, 100, Language::Message("Editors Font"))
          TextGadget(#Text_Editor_FontName, 10, GadgetY(#Frame_Editor_Color) + 20, 125, 34, Language::Message("Font"), #PB_Text_Center)
          StringGadget(#String_Editor_FontName, GadgetLocate::RightOf(#Text_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 125, 34, "", #PB_String_ReadOnly)
          ButtonGadget(#Button_Editor_FontSelect, GadgetLocate::RightOf(#String_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 50, 34, "...")
          TextGadget(#Text_Editor_FontSize, 10, GadgetLocate::DownOf(#Text_Editor_FontName, 5), 125, 34, Language::Message("Size"), #PB_Text_Center)
          StringGadget(#String_Editor_FontSize, GadgetLocate::RightOf(#Text_Editor_FontSize, 5), GadgetY(#Text_Editor_FontSize), 125, 34, "",  #PB_String_ReadOnly)
          
          FrameGadget(#Frame_Keyword_Bold, 5, GadgetLocate::DownOf(#Frame_Editor_Color, 5), 320, 55, Language::Message("Keywords"))
          CheckBoxGadget(#CheckBox_Keyword_Bold, 10, GadgetY(#Frame_Keyword_Bold) + 20, 300, 34, Language::Message("Keywords in bold"))
          
          FrameGadget(#Frame_Syntax_Colors, GadgetLocate::RightOf(#Frame_Date_Formatting, 5), GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 325, 490, Language::Message("Syntax coloring"))
          ScrollAreaGadget(#Scroll_Syntax_Colors, GadgetLocate::RightOf(#Frame_Date_Formatting, 10), GadgetY(#Frame_Syntax_Colors) + 20, 315, 465, 315, 940)
            
            StringGadget(#Text_DefaultForeColor, 5,5, 250,34, Language::Message("Normal Text"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_DefaultForeColor, 260, 5, 50, 32, SyntaxHighlighting::GetDefaultForeColor())
            
            StringGadget(#Text_DefaultBackColor, 5, GadgetLocate::DownOf(#Text_DefaultForeColor, 5), 250, 34, Language::Message("Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_DefaultBackColor, 260, GadgetY(#Text_DefaultBackColor), 50, 32, SyntaxHighlighting::GetDefaultBackColor())
            
            StringGadget(#Text_CaretForeColor, 5, GadgetLocate::DownOf(#Text_DefaultBackColor, 5), 250, 34, Language::Message("Caret"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_CaretForeColor, 260, GadgetY(#Text_CaretForeColor), 50, 32, SyntaxHighlighting::GetCaretForeColor())
            
            StringGadget(#Text_CaretBackColor, 5, GadgetLocate::DownOf(#Text_CaretForeColor, 5), 250, 34, Language::Message("Current line Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_CaretBackColor, 260, GadgetY(#Text_CaretBackColor), 50, 32, SyntaxHighlighting::GetCaretBackColor())
            
            StringGadget(#Text_SelectionForeColor, 5, GadgetLocate::DownOf(#Text_CaretBackColor, 5), 250, 34, Language::Message("Selection"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_SelectionForeColor, 260, GadgetY(#Text_SelectionForeColor), 50, 32, SyntaxHighlighting::GetSelectionForeColor())
            
            StringGadget(#Text_SelectionBackColor, 5, GadgetLocate::DownOf(#Text_SelectionForeColor, 5), 250, 34, Language::Message("Selection Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_SelectionBackColor, 260, GadgetY(#Text_SelectionBackColor), 50, 32, SyntaxHighlighting::GetSelectionBackColor())
            
            StringGadget(#Text_LineNumbersForeColor, 5, GadgetLocate::DownOf(#Text_SelectionBackColor, 5), 250, 34, Language::Message("Line Numbers"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_LineNumbersForeColor, 260, GadgetY(#Text_LineNumbersForeColor), 50, 32, SyntaxHighlighting::GetLineNumbersForeColor())
            
            StringGadget(#Text_LineNumbersBackColor, 5, GadgetLocate::DownOf(#Text_LineNumbersForeColor, 5), 250, 34, Language::Message("Line Numbers Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_LineNumbersBackColor, 260, GadgetY(#Text_LineNumbersBackColor), 50, 32, SyntaxHighlighting::GetLineNumbersBackColor())
            
            StringGadget(#Text_FoldMarginHiBackColor, 5, GadgetLocate::DownOf(#Text_LineNumbersBackColor, 5), 250, 34, Language::Message("Fold Margin Background") + " (Hi)",  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_FoldMarginHiBackColor, 260, GadgetY(#Text_FoldMarginHiBackColor), 50, 32, SyntaxHighlighting::GetFoldMarginHiBackColor())
            
            StringGadget(#Text_FoldMarginLoBackColor, 5, GadgetLocate::DownOf(#Text_FoldMarginHiBackColor, 5), 250, 34, Language::Message("Fold Margin Background") + " (Lo)",  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_FoldMarginLoBackColor, 260, GadgetY(#Text_FoldMarginLoBackColor), 50, 32, SyntaxHighlighting::GetFoldMarginLoBackColor())
            
            StringGadget(#Text_FoldMarkerForeColor, 5, GadgetLocate::DownOf(#Text_FoldMarginLoBackColor, 5), 250, 34, Language::Message("Fold Markers"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_FoldMarkerForeColor, 260, GadgetY(#Text_FoldMarkerForeColor), 50, 32, SyntaxHighlighting::GetFoldMarkerForeColor())
            
            StringGadget(#Text_FoldMarkerBackColor, 5, GadgetLocate::DownOf(#Text_FoldMarkerForeColor, 5), 250, 34, Language::Message("Fold Markers Background"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_FoldMarkerBackColor, 260, GadgetY(#Text_FoldMarkerBackColor), 50, 32, SyntaxHighlighting::GetFoldMarkerBackColor())
            
            StringGadget(#Text_LineHighlightColor, 5, GadgetLocate::DownOf(#Text_FoldMarkerBackColor, 5), 250, 34, Language::Message("Line Highlight"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_LineHighlightColor, 260, GadgetY(#Text_LineHighlightColor), 50, 32, SyntaxHighlighting::GetLineHighlightColor())
            
            StringGadget(#Text_KeywordsColor, 5, GadgetLocate::DownOf(#Text_LineHighlightColor, 5), 250, 34, Language::Message("Keywords"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_KeywordsColor, 260, GadgetY(#Text_KeywordsColor), 50, 32, SyntaxHighlighting::GetKeywordsColor())
            
            StringGadget(#Text_CommentColor, 5, GadgetLocate::DownOf(#Text_KeywordsColor, 5), 250, 34, Language::Message("Comments"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_CommentColor, 260, GadgetY(#Text_CommentColor), 50, 32, SyntaxHighlighting::GetCommentColor())
            
            StringGadget(#Text_LiteralStringColor, 5, GadgetLocate::DownOf(#Text_CommentColor, 5), 250, 34, Language::Message("Literal Strings"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_LiteralStringColor, 260, GadgetY(#Text_LiteralStringColor), 50, 32, SyntaxHighlighting::GetLiteralStringColor())
            
            StringGadget(#Text_NumbersColor, 5, GadgetLocate::DownOf(#Text_LiteralStringColor, 5), 250, 34, Language::Message("Numbers"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_NumbersColor, 260, GadgetY(#Text_NumbersColor), 50, 32, SyntaxHighlighting::GetNumbersColor())
            
            StringGadget(#Text_ConstantsColor, 5, GadgetLocate::DownOf(#Text_NumbersColor, 5), 250, 34, Language::Message("Constants"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_ConstantsColor, 260, GadgetY(#Text_ConstantsColor), 50, 32, SyntaxHighlighting::GetConstantsColor())
            
            StringGadget(#Text_ProceduresColor, 5, GadgetLocate::DownOf(#Text_ConstantsColor, 5), 250, 34, Language::Message("Procedures"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_ProceduresColor, 260, GadgetY(#Text_ProceduresColor), 50, 32, SyntaxHighlighting::GetProceduresColor())
            
            StringGadget(#Text_SeperatorsColor, 5, GadgetLocate::DownOf(#Text_ProceduresColor, 5), 250, 34, Language::Message("Separators"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_SeperatorsColor, 260, GadgetY(#Text_SeperatorsColor), 50, 32, SyntaxHighlighting::GetSeparatorsColor())
            
            StringGadget(#Text_OperatorsColor, 5, GadgetLocate::DownOf(#Text_SeperatorsColor, 5), 250, 34, Language::Message("Operators"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_OperatorsColor, 260, GadgetY(#Text_OperatorsColor), 50, 32, SyntaxHighlighting::GetOperatorsColor())
            
            StringGadget(#Text_TypesColor, 5, GadgetLocate::DownOf(#Text_OperatorsColor, 5), 250, 34, Language::Message("Types"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_TypesColor, 260, GadgetY(#Text_TypesColor), 50, 32, SyntaxHighlighting::GetTypesColor())
            
            StringGadget(#Text_ModulesColor, 5, GadgetLocate::DownOf(#Text_TypesColor, 5), 250, 34, Language::Message("Modules"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_ModulesColor, 260, GadgetY(#Text_ModulesColor), 50, 32, SyntaxHighlighting::GetModulesColor())
            
            StringGadget(#Text_PointersColor, 5, GadgetLocate::DownOf(#Text_ModulesColor, 5), 250, 34, Language::Message("Pointers"),  #PB_String_ReadOnly)
            ColorPreview::Gadget(#PreviewColor_PointersColor, 260, GadgetY(#Text_PointersColor), 50, 32, SyntaxHighlighting::GetPointersColor())
            
          CloseGadgetList() 
          
          ButtonGroup::Horizontal(#Button_Options_OK, #Button_Options_CANCEL, 660-150*2-10, 570, 150, 34, 5)
          
      CompilerEndSelect
      
      For ColorPreviewID = #PreviewColor_DefaultForeColor To #PreviewColor_PointersColor
        GadgetToolTip(ColorPreviewID, Language::Message("Click me"))
      Next
      
      SetGadgetText(#Frame_Project_SavePath, Language::Message("Project files folder"))
      SetGadgetText(#Button_Browse_Project_SavePath, Language::Message("Browse"))
      SetGadgetText(#String_Project_SavePath, Options::GetProjectSavePath())
      
      SetGadgetText(#Frame_User, Language::Message("User Informations"))
      SetGadgetText(#Text_User_Name, Language::Message("Name") + " :")
      SetGadgetText(#Text_User_NickName, Language::Message("AKA") + " :")
      SetGadgetText(#Text_User_Email, Language::Message("E-mail") + " :")
      
      SetGadgetText(#String_User_Name, Comments::GetUserName())
      SetGadgetText(#String_User_NickName, Comments::GetUserNickName())
      SetGadgetText(#String_User_Email, Comments::GetUserEmail())

      SetGadgetAttribute(#TrackBar_Tabulation_Size, #PB_TrackBar_Minimum, Comments::GetTabSizeMin())
      SetGadgetAttribute(#TrackBar_Tabulation_Size, #PB_TrackBar_Maximum, Comments::GetTabSizeMax())
      
      SetGadgetState(#TrackBar_Tabulation_Size, Comments::GetTabSize())
      SetGadgetText(#Text_Tabulation_Size, Str(Comments::GetTabSize()))
      
      If Comments::GetTabOption() = 0
        SetGadgetState(#Option_Use_NBSPACE_Tabulation, 1)
        DisableGadget(#TrackBar_Tabulation_Size, 0)
        DisableGadget(#Text_Tabulation_Size, 0)
      Else
        SetGadgetState(#Option_Use_ASCII_Tabulation, 1)
        DisableGadget(#TrackBar_Tabulation_Size, 1)
        DisableGadget(#Text_Tabulation_Size, 1)
      EndIf
 
      SetGadgetText(#Frame_Date_Formatting, Language::Message("Dates formatting"))
      
      If Comments::FirstDateMask()
        
        Repeat
          AddGadgetItem(#ComboBox_Date_Format, Comments::DateMaskIndex(), AdvancedDateFormatting::Format(Comments::GetDateMaskEx(), Date()))  
        Until Comments::NextDateMask() = #Null
        
      EndIf
      
      SetGadgetState(#ComboBox_Date_Format, Comments::GetSelectedDateMask())
      
      SetGadgetText(#String_Editor_FontName, Options::GetFontName())
      SetGadgetText(#String_Editor_FontSize, Str(Options::GetFontSize()))

      SetGadgetState(#CheckBox_Keyword_Bold, SyntaxHighlighting::GetKeywordsInBold())
      SetGadgetText(#Button_Options_OK, Language::Message("OK"))
      SetGadgetText(#Button_Options_CANCEL, Language::Message("Cancel"))
      
    EndIf
    
  EndProcedure
 
  Procedure CloseOptionsEditor(Mode.b)
    
    If Mode = 1
      
      Comments::SetUserName(GetGadgetText(#String_User_Name))
      Comments::SetUserNickName(GetGadgetText(#String_User_NickName))
      Comments::SetUserEmail(GetGadgetText(#String_User_Email))
      
      GoScintillaSetup(#Editor_Module_Primary, #False)
      GoScintillaSetup(#Editor_Module_Secondary, #False)
      GoScintillaSetup(#Editor_Source_Code, #False)
      
    EndIf
    
    For GadgetID = #PreviewColor_DefaultForeColor To #PreviewColor_ModulesColor
      ColorPreview::Free(GadgetID)
    Next
    
    Window::GetCurrentPositionSize(#OptionsWin)
    CloseWindow(#OptionsWin)
    HideWindow(#MainWin, #False)
    
  EndProcedure
  
  Procedure AskSavePath()
    
    NewPath.s = PathRequester(Language::Message("Select folder"), Options::GetProjectSavePath())
    
    If NewPath <> ""
      Options::SetProjectSavePath(NewPath)
      SetGadgetText(#String_Project_SavePath, Options::GetProjectSavePath())
    EndIf 
    
  EndProcedure
  
  Procedure AskNewFont()
    
    If FontRequester(Options::GetFontName(), Options::GetFontSize(), #PB_FontRequester_Effects)
      
      Options::SetFontName(SelectedFontName())
      Options::SetFontSize(SelectedFontSize())
      SetGadgetText(#String_Editor_FontName, Options::GetFontName())
      SetGadgetText(#String_Editor_FontSize, Str(Options::GetFontSize()))
      GoScintillaSetup(#Editor_Module_Primary, #False)
      GoScintillaSetup(#Editor_Module_Secondary, #False)
      GoScintillaSetup(#Editor_Source_Code, #False)
      
    EndIf 
    
  EndProcedure 
  
  Procedure AskNewColor(GadgetID)
    
    Select GadgetID
        
      Case #PreviewColor_DefaultForeColor
        CurrentColor.l = SyntaxHighlighting::GetDefaultForeColor()
        
      Case #PreviewColor_DefaultBackColor
        CurrentColor = SyntaxHighlighting::GetDefaultBackColor()
        
      Case #PreviewColor_CaretForeColor
        CurrentColor = SyntaxHighlighting::GetCaretForeColor()
        
      Case #PreviewColor_CaretBackColor
        CurrentColor = SyntaxHighlighting::GetCaretBackColor()
        
      Case #PreviewColor_SelectionForeColor
        CurrentColor = SyntaxHighlighting::GetSelectionForeColor()
        
      Case #PreviewColor_SelectionBackColor
        CurrentColor = SyntaxHighlighting::GetSelectionBackColor()
        
      Case #PreviewColor_LineNumbersForeColor
        CurrentColor = SyntaxHighlighting::GetLineNumbersForeColor()
        
      Case #PreviewColor_LineNumbersBackColor
        CurrentColor = SyntaxHighlighting::GetLineNumbersBackColor()
        
      Case #PreviewColor_FoldMarginHiBackColor
        CurrentColor = SyntaxHighlighting::GetFoldMarginHiBackColor()
        
      Case #PreviewColor_FoldMarginLoBackColor
        CurrentColor = SyntaxHighlighting::GetFoldMarginLoBackColor()
        
      Case #PreviewColor_FoldMarkerForeColor
        CurrentColor = SyntaxHighlighting::GetFoldMarkerForeColor()
        
      Case #PreviewColor_FoldMarkerBackColor
        CurrentColor = SyntaxHighlighting::GetFoldMarkerBackColor()
        
      Case #PreviewColor_LineHighlightColor
        CurrentColor = SyntaxHighlighting::GetLineHighlightColor()
        
      Case #PreviewColor_KeywordsColor
        CurrentColor = SyntaxHighlighting::GetKeywordsColor()
        
      Case #PreviewColor_CommentColor
        CurrentColor = SyntaxHighlighting::GetCommentColor()
        
      Case #PreviewColor_LiteralStringColor
        CurrentColor = SyntaxHighlighting::GetLiteralStringColor()
        
      Case #PreviewColor_NumbersColor
        CurrentColor = SyntaxHighlighting::GetNumbersColor()
        
      Case #PreviewColor_ConstantsColor
        CurrentColor = SyntaxHighlighting::GetConstantsColor()
        
      Case #PreviewColor_ProceduresColor
        CurrentColor = SyntaxHighlighting::GetProceduresColor()
        
      Case #PreviewColor_SeperatorsColor
        CurrentColor = SyntaxHighlighting::GetSeparatorsColor()
        
      Case #PreviewColor_OperatorsColor
        CurrentColor = SyntaxHighlighting::GetOperatorsColor()
        
      Case #PreviewColor_TypesColor
        CurrentColor = SyntaxHighlighting::GetTypesColor()
        
      Case #PreviewColor_ModulesColor
        CurrentColor = SyntaxHighlighting::GetModulesColor()
        
      Case #PreviewColor_PointersColor
        CurrentColor = SyntaxHighlighting::GetPointersColor()
        
    EndSelect 
    
    NewColor.l = ColorRequester(CurrentColor)
    
    If NewColor <> -1
      
      ColorPreview::CurrentColor(GadgetID, NewColor)
      
      Select GadgetID
          
        Case #PreviewColor_DefaultForeColor
          SyntaxHighlighting::SetDefaultForeColor(NewColor)
          
        Case #PreviewColor_DefaultBackColor
          SyntaxHighlighting::SetDefaultBackColor(NewColor)
          
        Case #PreviewColor_CaretForeColor
          SyntaxHighlighting::SetCaretForeColor(NewColor)
          
        Case #PreviewColor_CaretBackColor
          SyntaxHighlighting::SetCaretBackColor(NewColor)
          
        Case #PreviewColor_SelectionForeColor
          SyntaxHighlighting::SetSelectionForeColor(NewColor)
          
        Case #PreviewColor_SelectionBackColor
          SyntaxHighlighting::SetSelectionBackColor(NewColor)
          
        Case #PreviewColor_LineNumbersForeColor
          SyntaxHighlighting::SetLineNumbersForeColor(NewColor)
          
        Case #PreviewColor_LineNumbersBackColor
          SyntaxHighlighting::SetLineNumbersBackColor(NewColor)
          
        Case #PreviewColor_FoldMarginHiBackColor
          SyntaxHighlighting::SetFoldMarginHiBackColor(NewColor)
          
        Case #PreviewColor_FoldMarginLoBackColor
          SyntaxHighlighting::SetFoldMarginLoBackColor(NewColor)
          
        Case #PreviewColor_FoldMarkerForeColor
          SyntaxHighlighting::SetFoldMarkerForeColor(NewColor)
          
        Case #PreviewColor_FoldMarkerBackColor
          SyntaxHighlighting::SetFoldMarkerBackColor(NewColor)
          
        Case #PreviewColor_LineHighlightColor
          SyntaxHighlighting::SetLineHighlightColor(NewColor)
          
        Case #PreviewColor_KeywordsColor
          SyntaxHighlighting::SetKeywordsColor(NewColor)
          
        Case #PreviewColor_CommentColor
          SyntaxHighlighting::SetCommentColor(NewColor)
          
        Case #PreviewColor_LiteralStringColor
          SyntaxHighlighting::SetLiteralStringColor(NewColor)
          
        Case #PreviewColor_NumbersColor
          SyntaxHighlighting::SetNumbersColor(NewColor)
          
        Case #PreviewColor_ConstantsColor
          SyntaxHighlighting::SetConstantsColor(NewColor)
          
        Case #PreviewColor_ProceduresColor
          SyntaxHighlighting::SetProceduresColor(NewColor)
          
        Case #PreviewColor_SeperatorsColor
          SyntaxHighlighting::SetSeparatorsColor(NewColor)
          
        Case #PreviewColor_OperatorsColor
          SyntaxHighlighting::SetOperatorsColor(NewColor)
          
        Case #PreviewColor_TypesColor
          SyntaxHighlighting::SetTypesColor(NewColor)
          
        Case #PreviewColor_ModulesColor
          SyntaxHighlighting::SetModulesColor(NewColor)
          
        Case #PreviewColor_PointersColor
          SyntaxHighlighting::SetPointersColor(NewColor)
          
      EndSelect 
      
      GoScintillaSetup(#Editor_Module_Primary, #False)
      GoScintillaSetup(#Editor_Module_Secondary, #False)
      GoScintillaSetup(#Editor_Source_Code, #False)
      
    EndIf
    
  EndProcedure
  
  Procedure AskDateMaskEditor()
    
    NewMask.s = AdvancedDateFormatting::DateMaskEditor()
    
    If NewMask <> ""
      
      Comments::AddDateMask(NewMask)
      ClearGadgetItems(#ComboBox_Date_Format)
      
      If Comments::FirstDateMask()
        
        Repeat
          AddGadgetItem(#ComboBox_Date_Format, Comments::DateMaskIndex(), AdvancedDateFormatting::Format(Comments::GetDateMaskEx(), Date()))  
        Until Comments::NextDateMask() = #Null
        
        SetGadgetState(#ComboBox_Date_Format, Comments::GetSelectedDateMask())
        
      EndIf
      
    EndIf
    
  EndProcedure
  
  Procedure DeleteSelectedDateMask()
    
    If MessageRequester(Language::Message("Dev-Module - Warning"), ReplaceString(Language::Message("You are about to delete the date mask : %DATE_MASK%. Would you like to continue ?"), "%DATE_MASK%", Chr(34) + GetGadgetItemText(#ComboBox_Date_Format, GetGadgetState(#ComboBox_Date_Format)) + Chr(34)), #PB_MessageRequester_YesNo) = 6
      
      Comments::RemoveDateMask(GetGadgetState(#ComboBox_Date_Format))
      ClearGadgetItems(#ComboBox_Date_Format)
      
      If Comments::FirstDateMask()

        Repeat
          AddGadgetItem(#ComboBox_Date_Format, Comments::DateMaskIndex(), AdvancedDateFormatting::Format(Comments::GetDateMaskEx(), Date()))  
        Until Comments::NextDateMask() = #Null
        
        SetGadgetState(#ComboBox_Date_Format, Comments::GetSelectedDateMask())
        
      EndIf

      If Comments::DateMaskSize() = 0
        MessageRequester(Language::Message("Dev-Module - Error"), Language::Message("No Date Mask !"))
      EndIf
      
    EndIf
    
  EndProcedure
  
  Procedure GenerateCode()
    
    If From_GUI() = #True
      GoScintilla::SetText(#Editor_Source_Code, CodeGenerator::Generate())
      SetGadgetState(#Panel, 4)
    EndIf
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Initialize (Public) <<<<<
  
  Procedure Initialize()
    
    Path.s = GetPathPart(ProgramFilename())
    SetCurrentDirectory(Path)
    
    CompilerSelect #PB_Compiler_OS
        
      CompilerCase #PB_OS_Windows 
        UserPath.s = GetEnvironmentVariable("APPDATA") + #PathSep + "DevModule" + #PathSep
        
        If Left(UserPath, 2) <> "C:"
          UserPath = "C:" + #PathSep + UserPath
        EndIf
        
      CompilerCase #PB_OS_Linux
        UserPath.s = GetHomeDirectory() + ".devmodule" + #PathSep
        
      CompilerCase #PB_OS_MacOS
        UserPath.s = GetHomeDirectory() + ".devmodule" + #PathSep ; This might be wrong
        
    CompilerEndSelect
    
    BuiltPathDirectory::DoIt(UserPath)
    LanguagePath.s = GetPathPart(ProgramFilename()) + "language" + #PathSep
    
    CompilerIf #PB_Compiler_Debugger
      BuiltLanguageDefaultFiles(LanguagePath)
      
      CompilerIf #PB_Compiler_OS = #PB_OS_Linux
        LinuxDesktopFile_N_Icon()
      CompilerEndIf 
      
    CompilerEndIf 
    
    PrefFileName = UserPath + "DevModule.prefs"
    
    Window::Initialize(#MainWin, 50, 50, 885, 595, #False)
    Window::Initialize(#InternalStructureWin, 50, 50, 500, 405, #False)
    Window::Initialize(#OptionsWin, 50, 50, 660, 610, #False)
    
    If FindString(GetEnvironmentVariable("LANGUAGE"), "fr")
      Language::Initialize(LanguagePath, "Franais.xml", @Language_To_GUI()) 
    Else
      Language::Initialize(LanguagePath, "English.xml", @Language_To_GUI())
    EndIf
    
    Help::Initialize()
    Stats::Initialize()
    Options::Initialize()
    Comments::Initialize(#Version)
    AskPureBasicCompiler::Initialize()
    MatrixEncodedGenerator::Initialize()
    SyntaxHighlighting::Initialize()
    
    ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    ; <<<<< Chargement des icnes <<<<<
    
    UsePNGImageDecoder()
    
    Sticky::Initialize(#Button_PushPin, Path + "icons" + #PathSep,  "Push Pin 01 XP - 32.png", "Push Pin 02 XP - 32.png")
    LoadImage(#Button_New, Path + "icons" + #PathSep + "New XP - 32.png")
    LoadImage(#Button_Open, Path + "icons" + #PathSep + "Open XP - 32.png")
    LoadImage(#Button_Save, Path + "icons" + #PathSep + "Save XP - 32.png")
    LoadImage(#Button_Code, Path + "icons" + #PathSep + "Generate Code XP - 32.png")
    LoadImage(#Button_Brush, Path + "icons" + #PathSep + "Brush XP - 32.png")
    LoadImage(#Button_ClipBoard, Path + "icons" + #PathSep + "ClipBoard XP - 32.png")
    LoadImage(#Button_Language, Path + "icons" + #PathSep + "Language XP - 32.png")
    LoadImage(#Button_Options, Path + "icons" + #PathSep + "Option XP - 32.png")
    LoadImage(#Button_Quit, Path + "icons" + #PathSep + "Quit XP - 32.png")

    If FileSize(PrefFileName) = -1
      CreateDevModulePreferences()
    Else
      OpenDevModulePreferences()
    EndIf
    
    Language::LoadTableFile()
    Window::SanityCheck(#MainWin)
    Stats::Analysis()
    Help::SetPath(Language::GetFileName())
    
  EndProcedure
  
  Procedure CreateWindow()
    
    If Window::Open(#MainWin)
      
      CompilerSelect #PB_Compiler_OS
          
        CompilerCase #PB_OS_Windows 
          
          If InitScintilla() = 0
            CompilerIf #PB_Compiler_Debugger
              DebuggerError("Dev-Module - Fatal Error - Impossible To Initialize Scintilla !")
            CompilerElse 
              MessageRequester("Dev-Module - Fatal Error", "Impossible to Initialize Scintilla !")
              End
            CompilerEndIf
          EndIf
          
          PanelGadget(#Panel, 5, 5, Window::Width(#MainWin) - 65, Window::Height(#MainWin) - 10)
            
            AddGadgetItem(#Panel, 0, "") ; Dfinition du module
            
            TextGadget(#Text_Module_Name, 5, 7, 150, 20, "")
            StringGadget(#String_Module_Name, GadgetLocate::RightOf(#Text_Module_Name, 5), 5, 350, 25,"")
            
            PanelGadget(#Panel_Primary_Secondary, 5, GadgetLocate::DownOf(#String_Module_Name, 5), 805, 520)
              
              AddGadgetItem(#Panel_Primary_Secondary, 0, "")
              GoScintilla::Gadget(#Editor_Module_Primary, 5, 5, 590, 485, 0, GoScintilla::#AUTOSIZE_LINE_NUMBERS_MARGIN | GoScintilla::#KEYWORDS_CASE_SENSITIVE)
              OptionGroup::Vertical(#Option_Single_Instance, #Option_Off_Module_Instance, GadgetLocate::RightOf(#Editor_Module_Primary, 5), 5, 210, 30)
              
              AddGadgetItem(#Panel_Primary_Secondary, 1, "") 
              GoScintilla::Gadget(#Editor_Module_Secondary, 5, 5, 590, 485, 0, GoScintilla::#AUTOSIZE_LINE_NUMBERS_MARGIN | GoScintilla::#KEYWORDS_CASE_SENSITIVE)
              
              AddGadgetItem(#Panel_Primary_Secondary, 2, "") ; Dfinition des structures Internes
              
              FrameGadget(#Frame_Internal_Structure, 5, 5, 370, 485, "")
              ButtonGadget(#Button_Scan_Internal_Structure, 10, 30, 360, 30, "")
              ListViewGadget(#ListView_Internal_Structure, 10, GadgetLocate::DownOf(#Button_Scan_Internal_Structure, 5), 360, 420)
              
            CloseGadgetList()
            
            AddGadgetItem(#Panel, 1, "") ; Slection des mthodes Partie 1
            
            FrameGadget(#Frame_Basic_Methods, 5, 5, 400, 175, "")
            MethodGadgets(#CheckBox_Observator, GadgetX(#Frame_Basic_Methods) + 5, GadgetY(#Frame_Basic_Methods) + 20)
            MethodGadgets(#CheckBox_Mutator, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Observator))
            MethodGadgets(#CheckBox_Update, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Mutator))
            MethodGadgets(#CheckBox_Initialize, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Update))
            MethodGadgets(#CheckBox_Reset, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Initialize))
            MethodGadgets(#CheckBox_Clear, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Reset))

            FrameGadget(#Frame_Static_Array, 5, GadgetLocate::DownOf(#Frame_Basic_Methods), 400, 150, "")
            MethodGadgets(#CheckBox_QuickSortAscending, GadgetX(#Frame_Static_Array) + 5, GadgetY(#Frame_Static_Array) + 20)
            MethodGadgets(#CheckBox_QuickSortDescending, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortAscending))
            MethodGadgets(#CheckBox_LinearSearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortDescending))
            MethodGadgets(#CheckBox_BinarySearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_LinearSearch))
            MethodGadgets(#CheckBox_FixedLengthQueue, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_BinarySearch))
            
            FrameGadget(#Frame_Dynamic_Array, 5, GadgetLocate::DownOf(#Frame_Static_Array), 400, 150, "")
            MethodGadgets(#CheckBox_Sort_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetY(#Frame_Dynamic_Array) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Dynamic_Array))
            MethodGadgets(#CheckBox_Bin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Dynamic_Array))
            MethodGadgets(#CheckBox_Insert_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Bin_Search_Dynamic_Array))
            MethodGadgets(#CheckBox_Extract_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Insert_Dynamic_Array))
            
            FrameGadget(#Frame_Maths_Operator, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), 5, 400, 275, "")
            MethodGadgets(#CheckBox_Equal, GadgetX(#Frame_Maths_Operator) + 5, GadgetY(#Frame_Maths_Operator) + 20)
            MethodGadgets(#CheckBox_Plus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Equal))
            MethodGadgets(#CheckBox_Minus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Plus))
            MethodGadgets(#CheckBox_Product, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Minus))
            MethodGadgets(#CheckBox_Divide, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Product))
            MethodGadgets(#CheckBox_ProductByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Divide))
            MethodGadgets(#CheckBox_DivideByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_ProductByScalar))
            MethodGadgets(#CheckBox_Copy, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_DivideByScalar))
            MethodGadgets(#CheckBox_Compare, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Copy))
            MethodGadgets(#CheckBox_Swapping, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Compare))
            
            FrameGadget(#Frame_Debugging, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), GadgetLocate::DownOf(#Frame_Maths_Operator), 400, 50, "")
            MethodGadgets(#CheckBox_Debugging, GadgetX(#Frame_Debugging) + 5, GadgetY(#Frame_Debugging) + 20)
            
            AddGadgetItem(#Panel, 2, "") ; Slection des mthodes Partie 2
            
            FrameGadget(#Frame_XMLFile_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_XML_SetGet_Node, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetY(#Frame_XMLFile_Methods) + 20)
            MethodGadgets(#CheckBox_XML_SetGet_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Node))
            MethodGadgets(#CheckBox_XML_AddCollect_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Attribute))
            MethodGadgets(#CheckBox_XML_CreateLoad_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_AddCollect_Attribute))
            MethodGadgets(#CheckBox_XML_Catch_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_CreateLoad_File))
            
            FrameGadget(#Frame_JSONFile_Methods, 5, GadgetLocate::DownOf(#Frame_XMLFile_Methods), 400, 50, "")
            MethodGadgets(#CheckBox_JSON_CreateLoad_File, GadgetX(#Frame_JSONFile_Methods) + 5, GadgetY(#Frame_JSONFile_Methods) + 20)
            
            FrameGadget(#Frame_DataBaseFile_Methods, 5, GadgetLocate::DownOf(#Frame_JSONFile_Methods), 400, 250, "")
            MethodGadgets(#CheckBox_DataBase_CreateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetY(#Frame_DataBaseFile_Methods) + 20)
            MethodGadgets(#CheckBox_DataBase_DropTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CreateTable))
            MethodGadgets(#CheckBox_DataBase_TruncateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_DropTable))
            MethodGadgets(#CheckBox_DataBase_SelectTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_TruncateTable))
            MethodGadgets(#CheckBox_DataBase_CountRowInTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_SelectTable))
            MethodGadgets(#CheckBox_DataBase_OpenFile, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CountRowInTable))
            MethodGadgets(#CheckBox_DataBase_ModuleTo, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_OpenFile))
            MethodGadgets(#CheckBox_DataBase_ToModule, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ModuleTo))
            MethodGadgets(#CheckBox_DataBase_Vacuumize, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ToModule))
            
            FrameGadget(#Frame_Preference_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), 5, 400, 75, "")
            MethodGadgets(#CheckBox_Pref_Group, GadgetX(#Frame_Preference_Methods) + 5, GadgetY(#Frame_Preference_Methods) + 20)
            MethodGadgets(#CheckBox_Pref_File, GadgetX(#Frame_Preference_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Pref_Group))
            
            FrameGadget(#Frame_BinaryFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_Preference_Methods), 400, 125, "")
            MethodGadgets(#CheckBox_ReadWriteBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetY(#Frame_BinaryFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteBinary))
            MethodGadgets(#CheckBox_OpenBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateBinary))
            MethodGadgets(#CheckBox_ReadBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenBinary))
            
            FrameGadget(#Frame_EncodedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_BinaryFile_Methods), 400, 155, "")
            MethodGadgets(#CheckBox_ReadWriteEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetY(#Frame_EncodedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteEncoded))
            MethodGadgets(#CheckBox_OpenEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateEncoded))
            MethodGadgets(#CheckBox_ReadEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenEncoded))
            ExternalModuleGadgetGroup(#ComboBox_EncodedFile_Module, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadEncoded), 365)
            
            FrameGadget(#Frame_CryptedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_EncodedFile_Methods), 400, 155, "")
            MethodGadgets(#CheckBox_ReadWriteCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetY(#Frame_CryptedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteCrypted))
            MethodGadgets(#CheckBox_OpenCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateCrypted))
            MethodGadgets(#CheckBox_ReadCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenCrypted))
            ExternalModuleGadgetGroup(#ComboBox_CryptedFile_Module, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadCrypted), 365)
            
            AddGadgetItem(#Panel, 3, "") ; Slection des mthodes Partie 3
            
            FrameGadget(#Frame_Linked_List, 5, 5, 400, 425, "")
            MethodGadgets(#CheckBox_Sort_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetY(#Frame_Linked_List) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Linked_List))
            MethodGadgets(#CheckBox_Bin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Linked_List))
            MethodGadgets(#CheckBox_Add_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Bin_Search_Linked_List))
            MethodGadgets(#CheckBox_Select_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Add_Linked_List))
            MethodGadgets(#CheckBox_First_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Select_Linked_List))
            MethodGadgets(#CheckBox_Last_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_First_Linked_List))
            MethodGadgets(#CheckBox_Previous_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Last_Linked_List))
            MethodGadgets(#CheckBox_Next_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Previous_Linked_List))
            MethodGadgets(#CheckBox_Delete_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Next_Linked_List))
            MethodGadgets(#CheckBox_Reset_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Delete_Linked_List))
            MethodGadgets(#CheckBox_Clear_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Reset_Linked_List))
            MethodGadgets(#CheckBox_Index_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Clear_Linked_List))
            MethodGadgets(#CheckBox_Size_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Index_Linked_List))
            MethodGadgets(#CheckBox_Insert_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Size_Linked_List))
            MethodGadgets(#CheckBox_Extract_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Insert_Linked_List))
            
            AddGadgetItem(#Panel, 4, "") ; Code source
            GoScintilla::Gadget(#Editor_Source_Code, 5, 5, 805, 550, 0, GoScintilla::#AUTOSIZE_LINE_NUMBERS_MARGIN | GoScintilla::#KEYWORDS_CASE_SENSITIVE)
            
            AddGadgetItem(#Panel, 5, "")
            ComboBoxGadget(#ComboBox_Html_Help_Files, 5, 5, 805, 25)
            WebGadget(#Web_Html_Help_File, 5, 35, 805, 515, "")
            
          CloseGadgetList()

        CompilerCase #PB_OS_Linux
          
          PanelGadget(#Panel, 5, 5, Window::Width(#MainWin) - 65, Window::Height(#MainWin) - 10)
            
            AddGadgetItem(#Panel, 0, "") ; Dfinition du module
            
            TextGadget(#Text_Module_Name, 5, 7, 150, 20, "")
            StringGadget(#String_Module_Name, GadgetLocate::RightOf(#Text_Module_Name, 5), 5, 350, 35, "")
            
            PanelGadget(#Panel_Primary_Secondary, 5, GadgetLocate::DownOf(#String_Module_Name, 5), 810, 495)
              
              AddGadgetItem(#Panel_Primary_Secondary, 0, "")
              GoScintilla::Gadget(#Editor_Module_Primary, 5, 5, 590, 445, 0, GoScintilla::#AUTOSIZE_LINE_NUMBERS_MARGIN | GoScintilla::#KEYWORDS_CASE_SENSITIVE)
              OptionGroup::Vertical(#Option_Single_Instance, #Option_Off_Module_Instance, GadgetLocate::RightOf(#Editor_Module_Primary, 5), 5, 210, 30)
              
              AddGadgetItem(#Panel_Primary_Secondary, 1, "") 
              GoScintilla::Gadget(#Editor_Module_Secondary, 5, 5, 590, 445, 0, GoScintilla::#AUTOSIZE_LINE_NUMBERS_MARGIN | GoScintilla::#KEYWORDS_CASE_SENSITIVE)
              
              AddGadgetItem(#Panel_Primary_Secondary, 2, "") ; Dfinition des structures Internes
              
              FrameGadget(#Frame_Internal_Structure, 5, 5, 370, 445, "")
              ButtonGadget(#Button_Scan_Internal_Structure, 10, 30, 360, 30, "")
              ListViewGadget(#ListView_Internal_Structure, 10, GadgetLocate::DownOf(#Button_Scan_Internal_Structure, 5), 360, 375)
              
            CloseGadgetList()
            
            AddGadgetItem(#Panel, 1, "") ; Slection des mthodes Partie 1
            
            FrameGadget(#Frame_Basic_Methods, 5, 5, 400, 175, "")
            MethodGadgets(#CheckBox_Observator, GadgetX(#Frame_Basic_Methods) + 5, GadgetY(#Frame_Basic_Methods) + 20)
            MethodGadgets(#CheckBox_Mutator, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Observator))
            MethodGadgets(#CheckBox_Update, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Mutator))
            MethodGadgets(#CheckBox_Initialize, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Update))
            MethodGadgets(#CheckBox_Reset, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Initialize))
            MethodGadgets(#CheckBox_Clear, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Reset))

            FrameGadget(#Frame_Static_Array, 5, GadgetLocate::DownOf(#Frame_Basic_Methods), 400, 150, "")
            MethodGadgets(#CheckBox_QuickSortAscending, GadgetX(#Frame_Static_Array) + 5, GadgetY(#Frame_Static_Array) + 20)
            MethodGadgets(#CheckBox_QuickSortDescending, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortAscending))
            MethodGadgets(#CheckBox_LinearSearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortDescending))
            MethodGadgets(#CheckBox_BinarySearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_LinearSearch))
            MethodGadgets(#CheckBox_FixedLengthQueue, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_BinarySearch))
            

            
            FrameGadget(#Frame_Maths_Operator, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), 5, 400, 275, "")
            MethodGadgets(#CheckBox_Equal, GadgetX(#Frame_Maths_Operator) + 5, GadgetY(#Frame_Maths_Operator) + 20)
            MethodGadgets(#CheckBox_Plus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Equal))
            MethodGadgets(#CheckBox_Minus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Plus))
            MethodGadgets(#CheckBox_Product, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Minus))
            MethodGadgets(#CheckBox_Divide, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Product))
            MethodGadgets(#CheckBox_ProductByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Divide))
            MethodGadgets(#CheckBox_DivideByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_ProductByScalar))
            
            MethodGadgets(#CheckBox_Copy, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_DivideByScalar))
            MethodGadgets(#CheckBox_Compare, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Copy))
            MethodGadgets(#CheckBox_Swapping, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Compare))
            
            FrameGadget(#Frame_Debugging, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), GadgetLocate::DownOf(#Frame_Maths_Operator), 400, 50, "")
            MethodGadgets(#CheckBox_Debugging, GadgetX(#Frame_Debugging) + 5, GadgetY(#Frame_Debugging) + 20)
            
            AddGadgetItem(#Panel, 2, "") ; Slection des mthodes Partie 2
            
            FrameGadget(#Frame_XMLFile_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_XML_SetGet_Node, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetY(#Frame_XMLFile_Methods) + 20)
            MethodGadgets(#CheckBox_XML_SetGet_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Node))
            MethodGadgets(#CheckBox_XML_AddCollect_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Attribute))
            MethodGadgets(#CheckBox_XML_CreateLoad_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_AddCollect_Attribute))
            MethodGadgets(#CheckBox_XML_Catch_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_CreateLoad_File))
            
            FrameGadget(#Frame_JSONFile_Methods, 5, GadgetLocate::DownOf(#Frame_XMLFile_Methods), 400, 50, "")
            MethodGadgets(#CheckBox_JSON_CreateLoad_File, GadgetX(#Frame_JSONFile_Methods) + 5, GadgetY(#Frame_JSONFile_Methods) + 20)
            
            FrameGadget(#Frame_DataBaseFile_Methods, 5, GadgetLocate::DownOf(#Frame_JSONFile_Methods), 400, 250, "")
            MethodGadgets(#CheckBox_DataBase_CreateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetY(#Frame_DataBaseFile_Methods) + 20)
            MethodGadgets(#CheckBox_DataBase_DropTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CreateTable))
            MethodGadgets(#CheckBox_DataBase_TruncateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_DropTable))
            MethodGadgets(#CheckBox_DataBase_SelectTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_TruncateTable))
            MethodGadgets(#CheckBox_DataBase_CountRowInTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_SelectTable))
            MethodGadgets(#CheckBox_DataBase_OpenFile, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CountRowInTable))
            MethodGadgets(#CheckBox_DataBase_ModuleTo, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_OpenFile))
            MethodGadgets(#CheckBox_DataBase_ToModule, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ModuleTo))
            MethodGadgets(#CheckBox_DataBase_Vacuumize, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ToModule))
            
            FrameGadget(#Frame_Preference_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), 5, 400, 75, "")
            MethodGadgets(#CheckBox_Pref_Group, GadgetX(#Frame_Preference_Methods) + 5, GadgetY(#Frame_Preference_Methods) + 20)
            MethodGadgets(#CheckBox_Pref_File, GadgetX(#Frame_Preference_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Pref_Group))
            
            FrameGadget(#Frame_BinaryFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_Preference_Methods), 400, 125, "")
            MethodGadgets(#CheckBox_ReadWriteBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetY(#Frame_BinaryFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteBinary))
            MethodGadgets(#CheckBox_OpenBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateBinary))
            MethodGadgets(#CheckBox_ReadBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenBinary))
            
            FrameGadget(#Frame_EncodedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_BinaryFile_Methods), 400, 165, "")
            MethodGadgets(#CheckBox_ReadWriteEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetY(#Frame_EncodedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteEncoded))
            MethodGadgets(#CheckBox_OpenEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateEncoded))
            MethodGadgets(#CheckBox_ReadEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenEncoded))
            ExternalModuleGadgetGroup(#ComboBox_EncodedFile_Module, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadEncoded), 365)
            
            FrameGadget(#Frame_CryptedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_EncodedFile_Methods), 400, 165, "")
            MethodGadgets(#CheckBox_ReadWriteCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetY(#Frame_CryptedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteCrypted))
            MethodGadgets(#CheckBox_OpenCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateCrypted))
            MethodGadgets(#CheckBox_ReadCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenCrypted))
            ExternalModuleGadgetGroup(#ComboBox_CryptedFile_Module, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadCrypted), 365)
            
            AddGadgetItem(#Panel, 3, "") ; Slection des mthodes Partie 3
            
            FrameGadget(#Frame_Linked_List, 5, 5, 400, 425, "")
            MethodGadgets(#CheckBox_Sort_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetY(#Frame_Linked_List) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Linked_List))
            MethodGadgets(#CheckBox_Bin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Linked_List))
            MethodGadgets(#CheckBox_Add_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Bin_Search_Linked_List))
            MethodGadgets(#CheckBox_Select_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Add_Linked_List))
            MethodGadgets(#CheckBox_First_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Select_Linked_List))
            MethodGadgets(#CheckBox_Last_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_First_Linked_List))
            MethodGadgets(#CheckBox_Previous_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Last_Linked_List))
            MethodGadgets(#CheckBox_Next_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Previous_Linked_List))
            MethodGadgets(#CheckBox_Delete_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Next_Linked_List))
            MethodGadgets(#CheckBox_Reset_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Delete_Linked_List))
            MethodGadgets(#CheckBox_Clear_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Reset_Linked_List))
            MethodGadgets(#CheckBox_Index_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Clear_Linked_List))
            MethodGadgets(#CheckBox_Size_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Index_Linked_List))
            MethodGadgets(#CheckBox_Insert_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Size_Linked_List))
            MethodGadgets(#CheckBox_Extract_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Insert_Linked_List))
            
            FrameGadget(#Frame_Map, GadgetLocate::RightOf(#Frame_Linked_List, 5), 5, 400, 275, "")
            MethodGadgets(#CheckBox_Add_Map, GadgetX(#Frame_Map) + 5, GadgetY(#Frame_Map) + 20)
            MethodGadgets(#CheckBox_Clear_Map, GadgetX(#Frame_Map) + 5, GadgetLocate::DownOf(#CheckBox_Add_Map))
            MethodGadgets(#CheckBox_Size_Map, GadgetX(#Frame_Map) + 5, GadgetLocate::DownOf(#CheckBox_Clear_Map))
            MethodGadgets(#CheckBox_Reset_Map, GadgetX(#Frame_Map) + 5, GadgetLocate::DownOf(#CheckBox_Size_Map))
            MethodGadgets(#CheckBox_Delete_Map, GadgetX(#Frame_Map) + 5, GadgetLocate::DownOf(#CheckBox_Reset_Map))
            MethodGadgets(#CheckBox_Find_Map, GadgetX(#Frame_Map) + 5, GadgetLocate::DownOf(#CheckBox_Delete_Map))
            MethodGadgets(#CheckBox_Next_Map, GadgetX(#Frame_Map) + 5, GadgetLocate::DownOf(#CheckBox_Find_Map))
            MethodGadgets(#CheckBox_Key_Map, GadgetX(#Frame_Map) + 5, GadgetLocate::DownOf(#CheckBox_Next_Map))
            MethodGadgets(#CheckBox_Insert_Map, GadgetX(#Frame_Map) + 5, GadgetLocate::DownOf(#CheckBox_Key_Map))
            MethodGadgets(#CheckBox_Extract_Map, GadgetX(#Frame_Map) + 5, GadgetLocate::DownOf(#CheckBox_Insert_Map))
            
            FrameGadget(#Frame_Dynamic_Array, GadgetLocate::RightOf(#Frame_Linked_List, 5), GadgetLocate::DownOf(#Frame_Map), 400, 150, "")
            MethodGadgets(#CheckBox_Sort_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetY(#Frame_Dynamic_Array) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Dynamic_Array))
            MethodGadgets(#CheckBox_Bin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Dynamic_Array))
            MethodGadgets(#CheckBox_Insert_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Bin_Search_Dynamic_Array))
            MethodGadgets(#CheckBox_Extract_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Insert_Dynamic_Array))
            
            
            
            AddGadgetItem(#Panel, 4, "") ; Code source
            GoScintilla::Gadget(#Editor_Source_Code, 5, 5, 810, 535, 0, GoScintilla::#AUTOSIZE_LINE_NUMBERS_MARGIN | GoScintilla::#KEYWORDS_CASE_SENSITIVE)
            
            AddGadgetItem(#Panel, 5, "")
            ComboBoxGadget(#ComboBox_Html_Help_Files, 5, 5, 810, 30)
            WebGadget(#Web_Html_Help_File, 5, 40, 810, 505, "")
            
          CloseGadgetList()
          
        CompilerCase #PB_OS_MacOS ; Maybe the interface need some adjustment 
          
          PanelGadget(#Panel, 5, 5, Window::Width(#MainWin) - 65, Window::Height(#MainWin) - 10)
            
            AddGadgetItem(#Panel, 0, "") ; Dfinition du module
            
            TextGadget(#Text_Module_Name, 5, 7, 150, 20, "")
            StringGadget(#String_Module_Name, GadgetLocate::RightOf(#Text_Module_Name, 5), 5, 350, 35,"")
            
            PanelGadget(#Panel_Primary_Secondary, 5, GadgetLocate::DownOf(#String_Module_Name, 5), 810, 495)
              
              AddGadgetItem(#Panel_Primary_Secondary, 0, "")
              GoScintilla::Gadget(#Editor_Module_Primary, 5, 5, 590, 445, 0, GoScintilla::#AUTOSIZE_LINE_NUMBERS_MARGIN | GoScintilla::#KEYWORDS_CASE_SENSITIVE)
              OptionGroup::Vertical(#Option_Single_Instance, #Option_Off_Module_Instance, GadgetLocate::RightOf(#Editor_Module_Primary, 5), 5, 210, 30)
              
              AddGadgetItem(#Panel_Primary_Secondary, 1, "") 
              GoScintilla::Gadget(#Editor_Module_Secondary, 5, 5, 590, 445, 0, GoScintilla::#AUTOSIZE_LINE_NUMBERS_MARGIN | GoScintilla::#KEYWORDS_CASE_SENSITIVE)
              
              AddGadgetItem(#Panel_Primary_Secondary, 2, "") ; Dfinition des structures Internes
              
              FrameGadget(#Frame_Internal_Structure, 5, 5, 370, 445, "")
              ButtonGadget(#Button_Scan_Internal_Structure, 10, 30, 360, 30, "")
              ListViewGadget(#ListView_Internal_Structure, 10, GadgetLocate::DownOf(#Button_Scan_Internal_Structure, 5), 360, 375)
              
            CloseGadgetList()
            
            AddGadgetItem(#Panel, 1, "") ; Slection des mthodes Partie 1
            
            FrameGadget(#Frame_Basic_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_Observator, GadgetX(#Frame_Basic_Methods) + 5, GadgetY(#Frame_Basic_Methods) + 20)
            MethodGadgets(#CheckBox_Mutator, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Observator))
            MethodGadgets(#CheckBox_Update, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Mutator))
            MethodGadgets(#CheckBox_Initialize, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Update))
            MethodGadgets(#CheckBox_Reset, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Initialize))
            MethodGadgets(#CheckBox_Clear, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Reset))

            FrameGadget(#Frame_Static_Array, 5, GadgetLocate::DownOf(#Frame_Basic_Methods), 400, 150, "")
            MethodGadgets(#CheckBox_QuickSortAscending, GadgetX(#Frame_Static_Array) + 5, GadgetY(#Frame_Static_Array) + 20)
            MethodGadgets(#CheckBox_QuickSortDescending, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortAscending))
            MethodGadgets(#CheckBox_LinearSearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortDescending))
            MethodGadgets(#CheckBox_BinarySearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_LinearSearch))
            MethodGadgets(#CheckBox_FixedLengthQueue, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_BinarySearch))
            
            FrameGadget(#Frame_Dynamic_Array, 5, GadgetLocate::DownOf(#Frame_Static_Array), 400, 150, "")
            MethodGadgets(#CheckBox_Sort_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetY(#Frame_Dynamic_Array) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Dynamic_Array))
            MethodGadgets(#CheckBox_Bin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Dynamic_Array))
            MethodGadgets(#CheckBox_Insert_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Bin_Search_Dynamic_Array))
            MethodGadgets(#CheckBox_Extract_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Insert_Dynamic_Array))
            
            FrameGadget(#Frame_Maths_Operator, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), 5, 400, 275, "")
            MethodGadgets(#CheckBox_Equal, GadgetX(#Frame_Maths_Operator) + 5, GadgetY(#Frame_Maths_Operator) + 20)
            MethodGadgets(#CheckBox_Plus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Equal))
            MethodGadgets(#CheckBox_Minus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Plus))
            MethodGadgets(#CheckBox_Product, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Minus))
            MethodGadgets(#CheckBox_Divide, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Product))
            MethodGadgets(#CheckBox_ProductByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Divide))
            MethodGadgets(#CheckBox_DivideByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_ProductByScalar))
            
            MethodGadgets(#CheckBox_Copy, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_DivideByScalar))
            MethodGadgets(#CheckBox_Compare, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Copy))
            MethodGadgets(#CheckBox_Swapping, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Compare))
            
            FrameGadget(#Frame_Debugging, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), GadgetLocate::DownOf(#Frame_Maths_Operator), 400, 50, "")
            MethodGadgets(#CheckBox_Debugging, GadgetX(#Frame_Debugging) + 5, GadgetY(#Frame_Debugging) + 20)
            
            AddGadgetItem(#Panel, 2, "") ; Slection des mthodes Partie 2
            
            FrameGadget(#Frame_XMLFile_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_XML_SetGet_Node, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetY(#Frame_XMLFile_Methods) + 20)
            MethodGadgets(#CheckBox_XML_SetGet_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Node))
            MethodGadgets(#CheckBox_XML_AddCollect_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Attribute))
            MethodGadgets(#CheckBox_XML_CreateLoad_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_AddCollect_Attribute))
            MethodGadgets(#CheckBox_XML_Catch_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_CreateLoad_File))
            
            FrameGadget(#Frame_JSONFile_Methods, 5, GadgetLocate::DownOf(#Frame_XMLFile_Methods), 400, 50, "")
            MethodGadgets(#CheckBox_JSON_CreateLoad_File, GadgetX(#Frame_JSONFile_Methods) + 5, GadgetY(#Frame_JSONFile_Methods) + 20)
            
            FrameGadget(#Frame_DataBaseFile_Methods, 5, GadgetLocate::DownOf(#Frame_JSONFile_Methods), 400, 250, "")
            MethodGadgets(#CheckBox_DataBase_CreateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetY(#Frame_DataBaseFile_Methods) + 20)
            MethodGadgets(#CheckBox_DataBase_DropTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CreateTable))
            MethodGadgets(#CheckBox_DataBase_TruncateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_DropTable))
            MethodGadgets(#CheckBox_DataBase_SelectTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_TruncateTable))
            MethodGadgets(#CheckBox_DataBase_CountRowInTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_SelectTable))
            MethodGadgets(#CheckBox_DataBase_OpenFile, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CountRowInTable))
            MethodGadgets(#CheckBox_DataBase_ModuleTo, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_OpenFile))
            MethodGadgets(#CheckBox_DataBase_ToModule, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ModuleTo))
            MethodGadgets(#CheckBox_DataBase_Vacuumize, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ToModule))
            
            FrameGadget(#Frame_Preference_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), 5, 400, 75, "")
            MethodGadgets(#CheckBox_Pref_Group, GadgetX(#Frame_Preference_Methods) + 5, GadgetY(#Frame_Preference_Methods) + 20)
            MethodGadgets(#CheckBox_Pref_File, GadgetX(#Frame_Preference_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Pref_Group))
            
            FrameGadget(#Frame_BinaryFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_Preference_Methods), 400, 125, "")
            MethodGadgets(#CheckBox_ReadWriteBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetY(#Frame_BinaryFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteBinary))
            MethodGadgets(#CheckBox_OpenBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateBinary))
            MethodGadgets(#CheckBox_ReadBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenBinary))
            
            FrameGadget(#Frame_EncodedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_BinaryFile_Methods), 400, 165, "")
            MethodGadgets(#CheckBox_ReadWriteEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetY(#Frame_EncodedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteEncoded))
            MethodGadgets(#CheckBox_OpenEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateEncoded))
            MethodGadgets(#CheckBox_ReadEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenEncoded))
            ExternalModuleGadgetGroup(#ComboBox_EncodedFile_Module, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadEncoded), 365)
            
            FrameGadget(#Frame_CryptedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_EncodedFile_Methods), 400, 165, "")
            MethodGadgets(#CheckBox_ReadWriteCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetY(#Frame_CryptedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteCrypted))
            MethodGadgets(#CheckBox_OpenCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateCrypted))
            MethodGadgets(#CheckBox_ReadCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenCrypted))
            ExternalModuleGadgetGroup(#ComboBox_CryptedFile_Module, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadCrypted), 365)
            
            AddGadgetItem(#Panel, 3, "") ; Slection des mthodes Partie 3
            
            FrameGadget(#Frame_Linked_List, 5, 5, 400, 375, "")
            MethodGadgets(#CheckBox_Sort_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetY(#Frame_Linked_List) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Linked_List))
            MethodGadgets(#CheckBox_Bin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Linked_List))
            MethodGadgets(#CheckBox_Add_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Bin_Search_Linked_List))
            MethodGadgets(#CheckBox_Select_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Add_Linked_List))
            MethodGadgets(#CheckBox_First_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Select_Linked_List))
            MethodGadgets(#CheckBox_Last_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_First_Linked_List))
            MethodGadgets(#CheckBox_Previous_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Last_Linked_List))
            MethodGadgets(#CheckBox_Next_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Previous_Linked_List))
            MethodGadgets(#CheckBox_Delete_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Next_Linked_List))
            MethodGadgets(#CheckBox_Reset_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Delete_Linked_List))
            MethodGadgets(#CheckBox_Clear_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Reset_Linked_List))
            MethodGadgets(#CheckBox_Index_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Clear_Linked_List))
            MethodGadgets(#CheckBox_Size_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Index_Linked_List))
            MethodGadgets(#CheckBox_Insert_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Size_Linked_List))
            MethodGadgets(#CheckBox_Extract_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Insert_Linked_List))
            
            AddGadgetItem(#Panel, 4, "") ; Code source
            GoScintilla::Gadget(#Editor_Source_Code, 5, 5, 810, 535, 0, GoScintilla::#AUTOSIZE_LINE_NUMBERS_MARGIN | GoScintilla::#KEYWORDS_CASE_SENSITIVE)
            
            AddGadgetItem(#Panel, 5, "")
            ComboBoxGadget(#ComboBox_Html_Help_Files, 5, 5, 810, 30)
            WebGadget(#Web_Html_Help_File, 5, 40, 810, 505, "")
            
          CloseGadgetList()
         
      CompilerEndSelect
      
      GoScintillaSetup(#Editor_Module_Primary)
      GoScintillaSetup(#Editor_Module_Secondary)
      GoScintillaSetup(#Editor_Source_Code)
      
      ButtonImageGroup::Vertical(#Button_PushPin, #Button_Quit, GadgetLocate::RightOf(#Panel, 5), 5, 50, 50, 2)
      
      DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
      DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_CountRowInTable, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
      
      Help::SetComboBoxGadgetID(#ComboBox_Html_Help_Files)
      Help::SetWebGadgetID(#Web_Html_Help_File)
     
      MatrixGenerator::GadgetLinking(#String_Module_Name, 0)
      MatrixGenerator::GadgetLinking(#Editor_Module_Primary, 1)
      
      General::GadgetLinking(#ComboBox_EncodedFile_Module, 0)
      General::GadgetLinking(#ComboBox_CryptedFile_Module, 1)
      General::GadgetLinking(#ListView_Internal_Structure, 2)
      General::GadgetLinking(#Editor_Internal_Structure, 3)
      
      BindGadgetEvent(#Option_Use_NBSPACE_Tabulation, @TrackBarFollowOption())
      BindGadgetEvent(#Option_Use_ASCII_Tabulation, @TrackBarFollowOption())
      
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_Alt | #PB_Shortcut_F4, #Button_Quit)
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_F5, #Button_Code) 
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_F6, #Button_ClipBoard) 
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_F7, #Button_Brush)
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_Control | #PB_Shortcut_F1, #Shortcut_Matrix_Field_Setup)
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_Control | #PB_Shortcut_F2, #ShortCut_Matrix_Encoded)
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_Control | #PB_Shortcut_F3, #ShortCut_Stats_Windows)
      
      Sticky::ToggleStatus(#MainWin, #True)
      Language::To_GUI()
      General::FileModule_To_GUI()
      Help::ScanHelpFolder()
      To_GUI()
      
    Else
      
      ShutDown()
      
    EndIf

  EndProcedure
  
  Procedure EventManager()

    Repeat
      
      Select WaitWindowEvent()
          
        Case #PB_Event_CloseWindow
          
          Select EventWindow()
              
            Case #OptionsWin
             CloseOptionsEditor(0)
              
            Case #InternalStructureWin
              CloseInternalStructureEditor(0)
              
            Case #MainWin
              Shutdown()
              
          EndSelect
          
        Case #PB_Event_Menu
          
          Select EventMenu()
              
            Case #Button_ClipBoard
              SetClipboardText(GoScintilla::GetText(#Editor_Source_Code))
              
            Case #Button_Code
              CodeGenerator::Generate()
              
            Case #Button_Brush
              PastedCodeCleanUp(#Editor_Module_Primary)
              PastedCodeCleanUp(#Editor_Module_Secondary)
              
            Case #Shortcut_Matrix_Field_Setup
              MatrixGenerator()
              
            Case #ShortCut_Matrix_Encoded
              MatrixEncodedGenerator()
              
            Case #ShortCut_Stats_Windows
              Stats::OpenStatsWindow(#MainWin)
                
          EndSelect
          
        Case #PB_Event_Gadget
          
          Select EventGadget()
              
            Case #Button_Internal_Structure_OK
              CloseInternalStructureEditor(1)
 
            Case #Button_Internal_Structure_CANCEL
              CloseInternalStructureEditor(0)
              
            Case #Button_Options_OK
              CloseOptionsEditor(1)
              
            Case #Button_Options_CANCEL
              CloseOptionsEditor(0)
              
            Case #PreviewColor_DefaultForeColor To #PreviewColor_PointersColor
              If EventType() = #PB_EventType_LeftClick
                AskNewColor(EventGadget())
              EndIf
              
            Case #Button_Editor_FontSelect
              AskNewFont()
              
            Case #Button_Browse_Project_SavePath
              AskSavePath()
              
            Case #Button_Delete_Date_Mask
              DeleteSelectedDateMask()
              
            Case #Button_Date_Format_Editor
              AskDateMaskEditor()
              
            Case #ComboBox_Date_Format
              Comments::SelectDateMask(GetGadgetState(#ComboBox_Date_Format))
              
            Case #CheckBox_Keyword_Bold
              SyntaxHighlighting::SetKeywordsInBold(GetGadgetState(#CheckBox_Keyword_Bold))
              
            Case #Button_PushPin
              Sticky::ToggleStatus(#MainWin)
              
            Case #Button_New
              Refresh()
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_CountRowInTable, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_OpenFile, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Button_Open
              LoadDevModuleProjectFile()
              
            Case #Button_Save
              SaveFileDispatcher()
              
            Case #Button_Code
              GenerateCode()
                    
            Case #Button_Brush
              PastedCodeCleanUp(#Editor_Module_Primary)
              PastedCodeCleanUp(#Editor_Module_Secondary)
              
            Case #Button_Scan_Internal_Structure
              ScanInternalStructure()
              
            Case #ListView_Internal_Structure
              If EventType() = #PB_EventType_LeftDoubleClick
                If GetGadgetState(#ListView_Internal_Structure) >= 0
                  HideWindow(#MainWin, #True)
                  InternalStructureEditor()
                EndIf
              EndIf
              
            Case #Button_Add_EncodedFile_Module
              General::AddFileModule(0)
              
            Case #Button_Add_CryptedFile_Module
              General::AddFileModule(1)
              
            Case #Button_Remove_EncodedFile_Module
              General::RemoveFileModule(0)
              
            Case #Button_Remove_CryptedFile_Module
              General::RemoveFileModule(1)
              
            Case #Button_ClipBoard
              SetClipboardText(GoScintilla::GetText(#Editor_Source_Code))
              
            Case #Button_Language
              Language::Requester()
              Help::SetPath(Language::GetFileName())
              Help::ScanHelpFolder()
              
            Case #Button_Options
              OptionsEditor()

            Case #Button_Quit
              Shutdown()
 
            Case #Option_Single_Instance
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_CountRowInTable, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_OpenFile, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Option_Multiple_Instance_ID
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Option_Multiple_Instance_Index
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Option_Multiple_Instance_Name
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Option_Off_Module_Instance
              DependentGadgets::DoIt(1, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
              
            Case #ComboBox_Html_Help_Files
              Help::DirectlySelectHtmlFiles()
              
            Case #Option_Use_NBSPACE_Tabulation
              Comments::SetTabOption(0)
              DisableGadget(#TrackBar_Tabulation_Size, 0)
              DisableGadget(#Text_Tabulation_Size, 0)
              
            Case #Option_Use_ASCII_Tabulation
              Comments::SetTabOption(1)
              DisableGadget(#TrackBar_Tabulation_Size, 1)
              DisableGadget(#Text_Tabulation_Size, 1)
              
          EndSelect
          
      EndSelect
      
    ForEver
    
  EndProcedure
  
EndModule

; <<<<<<<<<<<<<<<<<<<<<<<<<<
; <<<<< FIN DU FICHIER <<<<<
; <<<<<<<<<<<<<<<<<<<<<<<<<<
; IDE Options = PureBasic 5.70 LTS (Linux - x64)
; Folding = vDAwA+-
; EnableXP
; CompileSourceDirectory