; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; Nom du projet : Gaming Maths Library
; Fichier : Matrix33 - Module.pb
; Version : 1.1.1
; Programmation : OK
; Programm par : Guillaume Saumure
; Alias : StarBootics
; Courriel : gsaumure@cgocable.ca
; Date : 20-05-2007
; Mise  jour : 05-06-2016
; Cod pour PureBasic V5.42
; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; Notes : Matrix33
;
; La matrice se prsente sous la forme suivante :
;
; lments Matrice --> Index du tableau 1D
;   3x3 lments   -->   9 case de 0  8
;  
;  [e11, e12, e13] -->    [0, 3, 6] 
;  [e21, e22, e23] -->    [1, 4, 7]
;  [e31, e32, e33] -->    [2, 5, 8]
;
; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

DeclareModule Matrix33
  
  Structure Matrix33

    e11.d
    e21.d
    e31.d
    
    e12.d
    e22.d
    e32.d
    
    e13.d
    e23.d
    e33.d

  EndStructure
  
  Declare.d GetE11(*MatrixA.Matrix33)
  Declare.d GetE21(*MatrixA.Matrix33)
  Declare.d GetE31(*MatrixA.Matrix33)
  Declare.d GetE12(*MatrixA.Matrix33)
  Declare.d GetE22(*MatrixA.Matrix33)
  Declare.d GetE32(*MatrixA.Matrix33)
  Declare.d GetE13(*MatrixA.Matrix33)
  Declare.d GetE23(*MatrixA.Matrix33)
  Declare.d GetE33(*MatrixA.Matrix33)
  
  Declare SetE11(*MatrixA.Matrix33, P_e11.d)
  Declare SetE21(*MatrixA.Matrix33, P_e21.d)
  Declare SetE31(*MatrixA.Matrix33, P_e31.d)
  Declare SetE12(*MatrixA.Matrix33, P_e12.d)
  Declare SetE22(*MatrixA.Matrix33, P_e22.d)
  Declare SetE32(*MatrixA.Matrix33, P_e32.d)
  Declare SetE13(*MatrixA.Matrix33, P_e13.d)
  Declare SetE23(*MatrixA.Matrix33, P_e23.d)
  Declare SetE33(*MatrixA.Matrix33, P_e33.d)
  
  Declare Reset(*MatrixA.Matrix33)
  Declare Copy(*MatrixA.Matrix33, *MatrixB.Matrix33)
  Declare Swapping(*MatrixA.Matrix33, *MatrixB.Matrix33)
  
  Declare Plus(*MatrixR.Matrix33, *MatrixA.Matrix33, *MatrixB.Matrix33)
  Declare Minus(*MatrixR.Matrix33, *MatrixA.Matrix33, *MatrixB.Matrix33)
  Declare Product(*MatrixR.Matrix33, *MatrixA.Matrix33, *MatrixB.Matrix33)
  Declare ProductEx(*MatrixR.Matrix33, *MatrixA.Matrix33, *MatrixB.Matrix33)
  Declare Multiply(*MatrixR.Matrix33, *MatrixA.Matrix33)
  Declare ProductByScalar(*MatrixR.Matrix33, *MatrixA.Matrix33, Scalar.d)
  Declare DivideByScalar(*MatrixR.Matrix33, *MatrixA.Matrix33, Scalar.d)
  Declare Zero(*MatrixA.Matrix33)
  Declare One(*MatrixA.Matrix33)
  Declare Identity(*Identity.Matrix33)
  Declare Transpose(*MatrixR.Matrix33, *MatrixA.Matrix33)
  Declare SelfTranspose(*MatrixA.Matrix33)
  
  Declare ReadMatrix33(FileID.i, *MatrixA.Matrix33)
  Declare WriteMatrix33(FileID.i, *MatrixA.Matrix33)
  Declare Randomize(*MatrixA.Matrix33, Min.d, Max.d, Resolution.l = 10000)
  
  Declare SetLine1(*MatrixA.Matrix33, P_e11.d, P_e12.d, P_e13.d)
  Declare SetLine2(*MatrixA.Matrix33, P_e21.d, P_e22.d, P_e23.d)
  Declare SetLine3(*MatrixA.Matrix33, P_e31.d, P_e32.d, P_e33.d)
  
  Declare.s FormatLine1(*MatrixA.Matrix33, Decimal.b = 3)
  Declare.s FormatLine2(*MatrixA.Matrix33, Decimal.b = 3)
  Declare.s FormatLine3(*MatrixA.Matrix33, Decimal.b = 3)
  
  Declare UnformatLine1(*MatrixA.Matrix33, P_MatrixLine.s)
  Declare UnformatLine2(*MatrixA.Matrix33, P_MatrixLine.s)
  Declare UnformatLine3(*MatrixA.Matrix33, P_MatrixLine.s)
  
  Declare.d Determinant(*MatrixA.Matrix33)
  Declare.b Inverse(*Inverse.Matrix33, *MatrixA.Matrix33)
  
  Declare Translation(*Trans.Matrix33, Trans_x.d, Trans_y.d)
  Declare Scale(*Scale.Matrix33, Scale_x.d, Scale_y.d)
  Declare MirrorX(*MirrorX.Matrix33)
  Declare MirrorY(*MirrorY.Matrix33)
  Declare Rotate(*Rotate.Matrix33, Theta.d)
  Declare LinearInterpolate(*MatrixR.Matrix33, *MatrixA.Matrix33, *MatrixB.Matrix33, P_t.d)
  Declare IsIntegerMatrix(*Matrix33A.Matrix33)
  
  CompilerIf Defined(Vector2, #PB_Module)
    Declare ApplyTransformation(*Transformation.Matrix33, *NewPoint.Vector2::Vector2, *Point.Vector2::Vector2)
  CompilerEndIf
  
  CompilerIf Defined(Vector3, #PB_Module)
    Declare ProductMatrixVector(*VectorR.Vector3::Vector3, *MatrixA.Matrix33, *VectorB.Vector3::Vector3)
    Declare ProductVectorMatrix(*VectorR.Vector3::Vector3, *VectorA.Vector3::Vector3, *MatrixB.Matrix33)
    Declare SetSkewSymmetric(*MatrixR.Matrix33, *VectorA.Vector3::Vector3)
    Declare TransposeTransform(*VectorR.Vector3::Vector3, *MatrixA.Matrix33, *VectorA.Vector3::Vector3)
  CompilerEndIf
  
EndDeclareModule

Module Matrix33
  
  Procedure.d GetE11(*MatrixA.Matrix33)
    
    ProcedureReturn *MatrixA\e11
  EndProcedure
  
  Procedure.d GetE21(*MatrixA.Matrix33)
    
    ProcedureReturn *MatrixA\e21
  EndProcedure
  
  Procedure.d GetE31(*MatrixA.Matrix33)
    
    ProcedureReturn *MatrixA\e31
  EndProcedure
  
  Procedure.d GetE12(*MatrixA.Matrix33)
    
    ProcedureReturn *MatrixA\e12
  EndProcedure
  
  Procedure.d GetE22(*MatrixA.Matrix33)
    
    ProcedureReturn *MatrixA\e22
  EndProcedure
  
  Procedure.d GetE32(*MatrixA.Matrix33)
    
    ProcedureReturn *MatrixA\e32
  EndProcedure
  
  Procedure.d GetE13(*MatrixA.Matrix33)
    
    ProcedureReturn *MatrixA\e13
  EndProcedure
  
  Procedure.d GetE23(*MatrixA.Matrix33)
    
    ProcedureReturn *MatrixA\e23
  EndProcedure
  
  Procedure.d GetE33(*MatrixA.Matrix33)
    
    ProcedureReturn *MatrixA\e33
  EndProcedure
  
  Procedure SetE11(*MatrixA.Matrix33, P_e11.d)
    
    *MatrixA\e11 = P_e11
    
  EndProcedure
  
  Procedure SetE21(*MatrixA.Matrix33, P_e21.d)
    
    *MatrixA\e21 = P_e21
    
  EndProcedure
  
  Procedure SetE31(*MatrixA.Matrix33, P_e31.d)
    
    *MatrixA\e31 = P_e31
    
  EndProcedure
  
  Procedure SetE12(*MatrixA.Matrix33, P_e12.d)
    
    *MatrixA\e12 = P_e12
    
  EndProcedure
  
  Procedure SetE22(*MatrixA.Matrix33, P_e22.d)
    
    *MatrixA\e22 = P_e22
    
  EndProcedure
  
  Procedure SetE32(*MatrixA.Matrix33, P_e32.d)
    
    *MatrixA\e32 = P_e32
    
  EndProcedure
  
  Procedure SetE13(*MatrixA.Matrix33, P_e13.d)
    
    *MatrixA\e13 = P_e13
    
  EndProcedure
  
  Procedure SetE23(*MatrixA.Matrix33, P_e23.d)
    
    *MatrixA\e23 = P_e23
    
  EndProcedure
  
  Procedure SetE33(*MatrixA.Matrix33, P_e33.d)
    
    *MatrixA\e33 = P_e33
    
  EndProcedure
  
  Procedure Reset(*MatrixA.Matrix33)
    
    *MatrixA\e11 = 0.0
    *MatrixA\e21 = 0.0
    *MatrixA\e31 = 0.0
    *MatrixA\e12 = 0.0
    *MatrixA\e22 = 0.0
    *MatrixA\e32 = 0.0
    *MatrixA\e13 = 0.0
    *MatrixA\e23 = 0.0
    *MatrixA\e33 = 0.0
    
  EndProcedure
  
  Procedure Copy(*MatrixA.Matrix33, *MatrixB.Matrix33)
    
    *MatrixB\e11 = *MatrixA\e11
    *MatrixB\e21 = *MatrixA\e21
    *MatrixB\e31 = *MatrixA\e31
    *MatrixB\e12 = *MatrixA\e12
    *MatrixB\e22 = *MatrixA\e22
    *MatrixB\e32 = *MatrixA\e32
    *MatrixB\e13 = *MatrixA\e13
    *MatrixB\e23 = *MatrixA\e23
    *MatrixB\e33 = *MatrixA\e33
    
  EndProcedure
  
  Procedure Swapping(*MatrixA.Matrix33, *MatrixB.Matrix33)
    
    Swap *MatrixA\e11, *MatrixB\e11
    Swap *MatrixA\e21, *MatrixB\e21
    Swap *MatrixA\e31, *MatrixB\e31
    Swap *MatrixA\e12, *MatrixB\e12
    Swap *MatrixA\e22, *MatrixB\e22
    Swap *MatrixA\e32, *MatrixB\e32
    Swap *MatrixA\e13, *MatrixB\e13
    Swap *MatrixA\e23, *MatrixB\e23
    Swap *MatrixA\e33, *MatrixB\e33
    
  EndProcedure
  
  Procedure Equal(*MatrixA.Matrix33, *MatrixB.Matrix33)
    
    *MatrixA\e11 = *MatrixB\e11
    *MatrixA\e21 = *MatrixB\e21
    *MatrixA\e31 = *MatrixB\e31
    *MatrixA\e12 = *MatrixB\e12
    *MatrixA\e22 = *MatrixB\e22
    *MatrixA\e32 = *MatrixB\e32
    *MatrixA\e13 = *MatrixB\e13
    *MatrixA\e23 = *MatrixB\e23
    *MatrixA\e33 = *MatrixB\e33
    
  EndProcedure
  
  Procedure Plus(*MatrixR.Matrix33, *MatrixA.Matrix33, *MatrixB.Matrix33)
    
    *MatrixR\e11 = *MatrixA\e11 + *MatrixB\e11
    *MatrixR\e21 = *MatrixA\e21 + *MatrixB\e21
    *MatrixR\e31 = *MatrixA\e31 + *MatrixB\e31
    *MatrixR\e12 = *MatrixA\e12 + *MatrixB\e12
    *MatrixR\e22 = *MatrixA\e22 + *MatrixB\e22
    *MatrixR\e32 = *MatrixA\e32 + *MatrixB\e32
    *MatrixR\e13 = *MatrixA\e13 + *MatrixB\e13
    *MatrixR\e23 = *MatrixA\e23 + *MatrixB\e23
    *MatrixR\e33 = *MatrixA\e33 + *MatrixB\e33
    
  EndProcedure
  
  Procedure Minus(*MatrixR.Matrix33, *MatrixA.Matrix33, *MatrixB.Matrix33)
    
    *MatrixR\e11 = *MatrixA\e11 - *MatrixB\e11
    *MatrixR\e21 = *MatrixA\e21 - *MatrixB\e21
    *MatrixR\e31 = *MatrixA\e31 - *MatrixB\e31
    *MatrixR\e12 = *MatrixA\e12 - *MatrixB\e12
    *MatrixR\e22 = *MatrixA\e22 - *MatrixB\e22
    *MatrixR\e32 = *MatrixA\e32 - *MatrixB\e32
    *MatrixR\e13 = *MatrixA\e13 - *MatrixB\e13
    *MatrixR\e23 = *MatrixA\e23 - *MatrixB\e23
    *MatrixR\e33 = *MatrixA\e33 - *MatrixB\e33
    
  EndProcedure
  
  Procedure Product(*MatrixR.Matrix33, *MatrixA.Matrix33, *MatrixB.Matrix33)
    
    *MatrixR\e11 = *MatrixA\e11 * *MatrixB\e11 + *MatrixA\e12 * *MatrixB\e21 + *MatrixA\e13 * *MatrixB\e31
    *MatrixR\e12 = *MatrixA\e11 * *MatrixB\e12 + *MatrixA\e12 * *MatrixB\e22 + *MatrixA\e13 * *MatrixB\e32
    *MatrixR\e13 = *MatrixA\e11 * *MatrixB\e13 + *MatrixA\e12 * *MatrixB\e23 + *MatrixA\e13 * *MatrixB\e33
    
    *MatrixR\e21 = *MatrixA\e21 * *MatrixB\e11 + *MatrixA\e22 * *MatrixB\e21 + *MatrixA\e23 * *MatrixB\e31
    *MatrixR\e22 = *MatrixA\e21 * *MatrixB\e12 + *MatrixA\e22 * *MatrixB\e22 + *MatrixA\e23 * *MatrixB\e32
    *MatrixR\e23 = *MatrixA\e21 * *MatrixB\e13 + *MatrixA\e22 * *MatrixB\e23 + *MatrixA\e23 * *MatrixB\e33
    
    *MatrixR\e31 = *MatrixA\e31 * *MatrixB\e11 + *MatrixA\e32 * *MatrixB\e21 + *MatrixA\e33 * *MatrixB\e31
    *MatrixR\e32 = *MatrixA\e31 * *MatrixB\e12 + *MatrixA\e32 * *MatrixB\e22 + *MatrixA\e33 * *MatrixB\e32
    *MatrixR\e33 = *MatrixA\e31 * *MatrixB\e13 + *MatrixA\e32 * *MatrixB\e23 + *MatrixA\e33 * *MatrixB\e33
    
  EndProcedure
  
  Procedure ProductEx(*MatrixR.Matrix33, *MatrixA.Matrix33, *MatrixB.Matrix33)
    
    Product(TempMatrix.Matrix33, *MatrixA, *MatrixB)
    Copy(TempMatrix, *MatrixR)
    
  EndProcedure
  
  Procedure Multiply(*MatrixR.Matrix33, *MatrixA.Matrix33)
    
    Product(TempMatrix.Matrix33, *MatrixR, *MatrixA)
    Copy(TempMatrix, *MatrixR)
    
  EndProcedure
  
  Procedure ProductByScalar(*MatrixR.Matrix33, *MatrixA.Matrix33, Scalar.d)
    
    *MatrixR\e11 = *MatrixA\e11 * Scalar
    *MatrixR\e21 = *MatrixA\e21 * Scalar
    *MatrixR\e31 = *MatrixA\e31 * Scalar
    *MatrixR\e12 = *MatrixA\e12 * Scalar
    *MatrixR\e22 = *MatrixA\e22 * Scalar
    *MatrixR\e32 = *MatrixA\e32 * Scalar
    *MatrixR\e13 = *MatrixA\e13 * Scalar
    *MatrixR\e23 = *MatrixA\e23 * Scalar
    *MatrixR\e33 = *MatrixA\e33 * Scalar
    
  EndProcedure
  
  Procedure DivideByScalar(*MatrixR.Matrix33, *MatrixA.Matrix33, Scalar.d)
    
    *MatrixR\e11 = *MatrixA\e11 / Scalar
    *MatrixR\e21 = *MatrixA\e21 / Scalar
    *MatrixR\e31 = *MatrixA\e31 / Scalar
    *MatrixR\e12 = *MatrixA\e12 / Scalar
    *MatrixR\e22 = *MatrixA\e22 / Scalar
    *MatrixR\e32 = *MatrixA\e32 / Scalar
    *MatrixR\e13 = *MatrixA\e13 / Scalar
    *MatrixR\e23 = *MatrixA\e23 / Scalar
    *MatrixR\e33 = *MatrixA\e33 / Scalar
    
  EndProcedure
  
  Procedure Zero(*MatrixA.Matrix33)
    
    *MatrixA\e11 = 0.0
    *MatrixA\e21 = 0.0
    *MatrixA\e31 = 0.0
    *MatrixA\e12 = 0.0
    *MatrixA\e22 = 0.0
    *MatrixA\e32 = 0.0
    *MatrixA\e13 = 0.0
    *MatrixA\e23 = 0.0
    *MatrixA\e33 = 0.0
    
  EndProcedure
  
  Procedure One(*MatrixA.Matrix33)
    
    *MatrixA\e11 = 1.0
    *MatrixA\e21 = 1.0
    *MatrixA\e31 = 1.0
    *MatrixA\e12 = 1.0
    *MatrixA\e22 = 1.0
    *MatrixA\e32 = 1.0
    *MatrixA\e13 = 1.0
    *MatrixA\e23 = 1.0
    *MatrixA\e33 = 1.0
    
  EndProcedure
  
  Procedure Identity(*Identity.Matrix33)
    
    *Identity\e11 = 1.0
    *Identity\e12 = 0.0
    *Identity\e13 = 0.0
    *Identity\e21 = 0.0
    *Identity\e22 = 1.0
    *Identity\e23 = 0.0
    *Identity\e31 = 0.0
    *Identity\e32 = 0.0
    *Identity\e33 = 1.0
    
  EndProcedure
  
  Procedure Transpose(*MatrixR.Matrix33, *MatrixA.Matrix33)
    
    *MatrixR\e11 = *MatrixA\e11
    *MatrixR\e21 = *MatrixA\e12
    *MatrixR\e31 = *MatrixA\e13
    *MatrixR\e12 = *MatrixA\e21
    *MatrixR\e22 = *MatrixA\e22
    *MatrixR\e32 = *MatrixA\e23
    *MatrixR\e13 = *MatrixA\e31
    *MatrixR\e23 = *MatrixA\e32
    *MatrixR\e33 = *MatrixA\e33
    
  EndProcedure
  
  Procedure SelfTranspose(*MatrixA.Matrix33)
    
    Swap *MatrixA\e12, *MatrixA\e21
    Swap *MatrixA\e13, *MatrixA\e31
    Swap *MatrixA\e23, *MatrixA\e32
    
  EndProcedure
  
  Procedure ReadMatrix33(FileID.i, *MatrixA.Matrix33)
    
    *MatrixA\e11 = ReadDouble(FileID)
    *MatrixA\e21 = ReadDouble(FileID)
    *MatrixA\e31 = ReadDouble(FileID)
    *MatrixA\e12 = ReadDouble(FileID)
    *MatrixA\e22 = ReadDouble(FileID)
    *MatrixA\e32 = ReadDouble(FileID)
    *MatrixA\e13 = ReadDouble(FileID)
    *MatrixA\e23 = ReadDouble(FileID)
    *MatrixA\e33 = ReadDouble(FileID)
    
  EndProcedure
  
  Procedure WriteMatrix33(FileID.i, *MatrixA.Matrix33)
    
    WriteDouble(FileID, *MatrixA\e11)
    WriteDouble(FileID, *MatrixA\e21)
    WriteDouble(FileID, *MatrixA\e31)
    WriteDouble(FileID, *MatrixA\e12)
    WriteDouble(FileID, *MatrixA\e22)
    WriteDouble(FileID, *MatrixA\e32)
    WriteDouble(FileID, *MatrixA\e13)
    WriteDouble(FileID, *MatrixA\e23)
    WriteDouble(FileID, *MatrixA\e33)
    
  EndProcedure
  
  Procedure Randomize(*MatrixA.Matrix33, Min.d, Max.d, Resolution.l = 10000)
    
    *MatrixA\e11 = (Min + (Max - Min) * Random(Resolution) / Resolution)
    *MatrixA\e12 = (Min + (Max - Min) * Random(Resolution) / Resolution)
    *MatrixA\e13 = (Min + (Max - Min) * Random(Resolution) / Resolution)
    *MatrixA\e21 = (Min + (Max - Min) * Random(Resolution) / Resolution)
    *MatrixA\e22 = (Min + (Max - Min) * Random(Resolution) / Resolution)
    *MatrixA\e23 = (Min + (Max - Min) * Random(Resolution) / Resolution)
    *MatrixA\e31 = (Min + (Max - Min) * Random(Resolution) / Resolution)
    *MatrixA\e32 = (Min + (Max - Min) * Random(Resolution) / Resolution)
    *MatrixA\e33 = (Min + (Max - Min) * Random(Resolution) / Resolution)
    
  EndProcedure
  
  Procedure SetLine1(*MatrixA.Matrix33, P_e11.d, P_e12.d, P_e13.d)
    
    *MatrixA\e11 = P_e11
    *MatrixA\e12 = P_e12
    *MatrixA\e13 = P_e13
    
  EndProcedure
  
  Procedure SetLine2(*MatrixA.Matrix33, P_e21.d, P_e22.d, P_e23.d)
    
    *MatrixA\e21 = P_e21
    *MatrixA\e22 = P_e22
    *MatrixA\e23 = P_e23
    
  EndProcedure
  
  Procedure SetLine3(*MatrixA.Matrix33, P_e31.d, P_e32.d, P_e33.d)
    
    *MatrixA\e31 = P_e31
    *MatrixA\e32 = P_e32
    *MatrixA\e33 = P_e33
    
  EndProcedure
  
  Procedure.s FormatLine1(*MatrixA.Matrix33, Decimal.b = 3)
    
    ProcedureReturn "[" + StrD(*MatrixA\e11, Decimal) + " : " + StrD(*MatrixA\e12, Decimal) + " : " + StrD(*MatrixA\e13, Decimal) + "]"
  EndProcedure
  
  Procedure.s FormatLine2(*MatrixA.Matrix33, Decimal.b = 3)
    
    ProcedureReturn "[" + StrD(*MatrixA\e21, Decimal) + " : " + StrD(*MatrixA\e22, Decimal) + " : " + StrD(*MatrixA\e23, Decimal) + "]"
  EndProcedure
  
  Procedure.s FormatLine3(*MatrixA.Matrix33, Decimal.b = 3)
    
    ProcedureReturn "[" + StrD(*MatrixA\e31, Decimal) + " : " + StrD(*MatrixA\e32, Decimal) + " : " + StrD(*MatrixA\e33, Decimal) + "]"
  EndProcedure
  
  Procedure UnformatLine1(*MatrixA.Matrix33, P_MatrixLine.s)
    
    *MatrixA\e11 = ValD(StringField(StringField(StringField(P_MatrixLine, 2, "["), 1, "]"), 1, ":"))
    *MatrixA\e12 = ValD(StringField(StringField(StringField(P_MatrixLine, 2, "["), 1, "]"), 2, ":"))
    *MatrixA\e13 = ValD(StringField(StringField(StringField(P_MatrixLine, 2, "["), 1, "]"), 3, ":"))
    
  EndProcedure
  
  Procedure UnformatLine2(*MatrixA.Matrix33, P_MatrixLine.s)
    
    *MatrixA\e21 = ValD(StringField(StringField(StringField(P_MatrixLine, 2, "["), 1, "]"), 1, ":"))
    *MatrixA\e22 = ValD(StringField(StringField(StringField(P_MatrixLine, 2, "["), 1, "]"), 2, ":"))
    *MatrixA\e23 = ValD(StringField(StringField(StringField(P_MatrixLine, 2, "["), 1, "]"), 3, ":"))
    
  EndProcedure
  
  Procedure UnformatLine3(*MatrixA.Matrix33, P_MatrixLine.s)
    
    *MatrixA\e31 = ValD(StringField(StringField(StringField(P_MatrixLine, 2, "["), 1, "]"), 1, ":"))
    *MatrixA\e32 = ValD(StringField(StringField(StringField(P_MatrixLine, 2, "["), 1, "]"), 2, ":"))
    *MatrixA\e33 = ValD(StringField(StringField(StringField(P_MatrixLine, 2, "["), 1, "]"), 3, ":"))
    
  EndProcedure
  
  Procedure.d Determinant(*MatrixA.Matrix33)
    
    ProcedureReturn (*MatrixA\e11 * (*MatrixA\e22 * *MatrixA\e33 - *MatrixA\e23) * *MatrixA\e32) - *MatrixA\e12 * (*MatrixA\e21 * *MatrixA\e33 - *MatrixA\e23 * *MatrixA\e31) + *MatrixA\e13 * (*MatrixA\e21 * *MatrixA\e32 - *MatrixA\e22 * *MatrixA\e31)
    
  EndProcedure
  
  Procedure.b Inverse(*Inverse.Matrix33, *MatrixA.Matrix33)
    
    Copy(*MatrixA, CopyMatrixA.Matrix33)
    
    Cofactor_Determinant.d = Determinant(CopyMatrixA)
    
    If Cofactor_Determinant <> 0.0
      
      *Inverse\e11 = (CopyMatrixA\e22 * CopyMatrixA\e33 - CopyMatrixA\e23 * CopyMatrixA\e32) / Cofactor_Determinant
      *Inverse\e12 = (CopyMatrixA\e13 * CopyMatrixA\e32 - CopyMatrixA\e12 * CopyMatrixA\e33) / Cofactor_Determinant
      *Inverse\e13 = (CopyMatrixA\e12 * CopyMatrixA\e23 - CopyMatrixA\e13 * CopyMatrixA\e22) / Cofactor_Determinant
      
      *Inverse\e21 = (CopyMatrixA\e23 * CopyMatrixA\e31 - CopyMatrixA\e21 * CopyMatrixA\e33) / Cofactor_Determinant
      *Inverse\e22 = (CopyMatrixA\e11 * CopyMatrixA\e33 - CopyMatrixA\e13 * CopyMatrixA\e31) / Cofactor_Determinant
      *Inverse\e23 = (CopyMatrixA\e13 * CopyMatrixA\e21 - CopyMatrixA\e11 * CopyMatrixA\e23) / Cofactor_Determinant
      
      *Inverse\e31 = (CopyMatrixA\e21 * CopyMatrixA\e32 - CopyMatrixA\e22 * CopyMatrixA\e31) / Cofactor_Determinant
      *Inverse\e32 = (CopyMatrixA\e12 * CopyMatrixA\e31 - CopyMatrixA\e11 * CopyMatrixA\e32) / Cofactor_Determinant
      *Inverse\e33 = (CopyMatrixA\e11 * CopyMatrixA\e22 - CopyMatrixA\e12 * CopyMatrixA\e21) / Cofactor_Determinant
      
      Result.b = #True
      
    Else 
      
      *Inverse\e11 = 1.0
      *Inverse\e12 = 0.0
      *Inverse\e13 = 0.0
      *Inverse\e21 = 0.0
      *Inverse\e22 = 1.0
      *Inverse\e23 = 0.0
      *Inverse\e31 = 0.0
      *Inverse\e32 = 0.0
      *Inverse\e33 = 1.0
      Result = #False 
      
    EndIf 
    
    ProcedureReturn Result 
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Le mutateur pour la Translation <<<<<
  
  Procedure Translation(*Trans.Matrix33, Trans_x.d, Trans_y.d)
    
    SetLine1(*Trans, 1, 0, Trans_x)
    SetLine2(*Trans, 0, 1, Trans_y)
    SetLine3(*Trans, 0, 0, 1)
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Le mutateur pour le Scale <<<<<
  
  Procedure Scale(*Scale.Matrix33, Scale_x.d, Scale_y.d)
    
    SetLine1(*Scale, Abs(Scale_x), 0, 0)
    SetLine2(*Scale, 0, Abs(Scale_y), 0)
    SetLine3(*Scale, 0, 0, 1)
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Le mutateur pour le MirrorX 2D >>>>>
  
  Procedure MirrorX(*MirrorX.Matrix33)
    
    SetLine1(*MirrorX, 1, 0, 0)
    SetLine2(*MirrorX, 0, -1, 0)
    SetLine3(*MirrorX, 0, 0, 1)
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Le mutateur pour le MirrorY 2D >>>>>
  
  Procedure MirrorY(*MirrorY.Matrix33)
    
    SetLine1(*MirrorY, -1, 0, 0)
    SetLine2(*MirrorY, 0, 1, 0)
    SetLine3(*MirrorY, 0, 0, 1)
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Le mutateur pour le Rotate 2D <<<<<
  
  Procedure Rotate(*Rotate.Matrix33, Theta.d)
    
    Protected Cos.d, Sin.d 
    
    !FLD qword [p.v_Theta]
    !FSINCOS
    !FSTP qword [p.v_Cos]
    !FSTP qword [p.v_Sin] 
    
    SetLine1(*Rotate, Cos, -Sin, 0)
    SetLine2(*Rotate, Sin, Cos, 0)
    SetLine3(*Rotate, 0, 0, 1)
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< L'oprateur LinearInterpolate <<<<<
  
  Procedure LinearInterpolate(*MatrixR.Matrix33, *MatrixA.Matrix33, *MatrixB.Matrix33, P_t.d)
    
    ProductByScalar(LerpMatrix00.Matrix33, *MatrixA, (1 - P_t))
    ProductByScalar(LerpMatrix01.Matrix33, *MatrixB, P_t)
    Plus(*MatrixR, LerpMatrix00, LerpMatrix01)
    
  EndProcedure
  
    ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Est une matrice d'entier ? <<<<<
  
  Procedure IsIntegerMatrix(*Matrix33A.Matrix33)
    
    If *Matrix33A\e11 <> Int(*Matrix33A\e11)
      ProcedureReturn #False
    EndIf
    
    If *Matrix33A\e12 <> Int(*Matrix33A\e12)
      ProcedureReturn #False
    EndIf
    
    If *Matrix33A\e13 <> Int(*Matrix33A\e13)
      ProcedureReturn #False
    EndIf

    
    If *Matrix33A\e21 <> Int(*Matrix33A\e21)
      ProcedureReturn #False
    EndIf
    
    If *Matrix33A\e22 <> Int(*Matrix33A\e22)
      ProcedureReturn #False
    EndIf
    
    If *Matrix33A\e23 <> Int(*Matrix33A\e23)
      ProcedureReturn #False
    EndIf

    
    If *Matrix33A\e31 <> Int(*Matrix33A\e31)
      ProcedureReturn #False
    EndIf
    
    If *Matrix33A\e32 <> Int(*Matrix33A\e32)
      ProcedureReturn #False
    EndIf
    
    If *Matrix33A\e33 <> Int(*Matrix33A\e33)
      ProcedureReturn #False
    EndIf

    ProcedureReturn #True
  EndProcedure
  
  
  
  
  CompilerIf Defined(Vector2, #PB_Module)
    
    Procedure ApplyTransformation(*Transformation.Matrix33, *NewPoint.Vector2::Vector2, *Point.Vector2::Vector2)
      
      Vector3::SetI(PointHomo.Vector3::Vector3, Vector2::GetI(*Point))
      Vector3::SetJ(PointHomo.Vector3::Vector3, Vector2::GetJ(*Point))
      Vector3::SetK(PointHomo.Vector3::Vector3, 1.0)
      ProductMatrixVector(NewPointHomo.Vector3::Vector3, *Transformation, PointHomo)
      Vector2::SetI(*NewPoint, Vector3::GetI(NewPointHomo))
      Vector2::SetJ(*NewPoint, Vector3::GetJ(NewPointHomo))
      
    EndProcedure
    
  CompilerEndIf
  
  CompilerIf Defined(Vector3, #PB_Module)
    
    ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    ; <<<<< L'oprateur Product Matrix Vector : R = A * B <<<<<
    ; <<<<< Le VectorR doit tre diffrent du VectorB.    <<<<<
    
    Procedure ProductMatrixVector(*VectorR.Vector3::Vector3, *MatrixA.Matrix33, *VectorB.Vector3::Vector3)
      
      *VectorR\i = *MatrixA\e11 * *VectorB\i + *MatrixA\e12 * *VectorB\j + *MatrixA\e13 * *VectorB\k
      *VectorR\j = *MatrixA\e21 * *VectorB\i + *MatrixA\e22 * *VectorB\j + *MatrixA\e23 * *VectorB\k
      *VectorR\k = *MatrixA\e31 * *VectorB\i + *MatrixA\e32 * *VectorB\j + *MatrixA\e33 * *VectorB\k
      
    EndProcedure
    
    ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    ; <<<<< L'oprateur Product Vector Matrix : R = A * B <<<<<
    ; <<<<< Le VectorR doit tre diffrent du VectorA.    <<<<<
    
    Procedure ProductVectorMatrix(*VectorR.Vector3::Vector3, *VectorA.Vector3::Vector3, *MatrixB.Matrix33)
      
      *VectorR\i = *MatrixB\e11 * *VectorA\i + *MatrixB\e21 * *VectorA\j + *MatrixB\e31 * *VectorA\k
      *VectorR\j = *MatrixB\e12 * *VectorA\i + *MatrixB\e22 * *VectorA\j + *MatrixB\e32 * *VectorA\k
      *VectorR\k = *MatrixB\e13 * *VectorA\i + *MatrixB\e23 * *VectorA\j + *MatrixB\e33 * *VectorA\k
      
    EndProcedure
    
    Procedure SetSkewSymmetric(*MatrixR.Matrix33, *VectorA.Vector3::Vector3)
      
      *MatrixR\e11 = 0.0
      *MatrixR\e22 = 0.0
      *MatrixR\e33 = 0.0
      
      *MatrixR\e12 = -*VectorA\k
      *MatrixR\e13 = *VectorA\j
      
      *MatrixR\e21 = *VectorA\k
      *MatrixR\e23 = -*VectorA\i
      
      *MatrixR\e31 = -*VectorA\j
      *MatrixR\e32 = *VectorA\i 
      
    EndProcedure
    
    Procedure TransposeTransform(*VectorR.Vector3::Vector3, *MatrixA.Matrix33, *VectorB.Vector3::Vector3)
      
      *VectorR\i = *MatrixA\e11 * *VectorB\i + *MatrixA\e21 * *VectorB\j + *MatrixA\e31 * *VectorB\k
      *VectorR\j = *MatrixA\e12 * *VectorB\i + *MatrixA\e22 * *VectorB\j + *MatrixA\e32 * *VectorB\k
      *VectorR\k = *MatrixA\e13 * *VectorB\i + *MatrixA\e23 * *VectorB\j + *MatrixA\e33 * *VectorB\k
      
    EndProcedure
    
  CompilerEndIf
  
EndModule

CompilerIf #PB_Compiler_IsMainFile
  
;   Matrix33::SetLine1(MatA.Matrix33::Matrix33, -6,-3,-2)
;   Matrix33::SetLine2(MatA.Matrix33::Matrix33, 9,4,3)
;   Matrix33::SetLine3(MatA.Matrix33::Matrix33, -1,5,0)
;   
;   Matrix33::Inverse(Inverse.Matrix33::Matrix33, MatA)
;   
; 
;   Vector3::Update(VecA.Vector3::Vector3, 50, 51, 45)
;   
;   Matrix33::ProductMatrixVector(VecR.Vector3::Vector3, MatA, VecA)
;   Debug Vector3::Format(VecA)
;   Debug Vector3::Format(VecR)
;   Debug ""
;   
;   Matrix33::ProductMatrixVector(VecB.Vector3::Vector3, Inverse, VecR)
;   
;   Debug Vector3::Format(VecB)
;   
  
  Matrix33::Rotate(Rotate.Matrix33::Matrix33, Radian(45))
  
  Debug "Matrice de rotation 2D"
  Debug Matrix33::FormatLine1(Rotate)
  Debug Matrix33::FormatLine2(Rotate)
  Debug Matrix33::FormatLine3(Rotate)
  
  
CompilerEndIf


; ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; ; <<<<< L'oprateur Tensor <<<<<
; 
; Macro TensorMatrix33(MatrixA, P_e11, P_e22, P_e33, P_e12, P_e13, P_e23)
;   
;   SetMatrix33Line1(MatrixA, P_e11, P_e12, P_e13)
;   SetMatrix33Line2(MatrixA, P_e12, P_e22, P_e23)
;   SetMatrix33Line3(MatrixA, P_e13, P_e23, P_e33)
;   
; EndMacro
; 
; ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; ; <<<<< L'oprateur SphereInertiaTensorMatrix33 <<<<<
; 
; Macro SphereInertiaTensorMatrix33(MatrixA, Mass, Radius)
;   
;   SetMatrix33Line1(MatrixA, (2.0 * Mass * Radius * Radius) / 5.0, 0.0, 0.0)
;   SetMatrix33Line2(MatrixA, 0.0, (2.0 * Mass * Radius * Radius) / 5.0, 0.0)
;   SetMatrix33Line3(MatrixA, 0.0, 0.0, (2.0 * Mass * Radius * Radius) / 5.0)
;   
; EndMacro
; 
; ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; ; <<<<< L'oprateur CylinderInertiaTensorMatrix33 <<<<<
; ; <<<<< La longueur est aligne selon l'axe des Z <<<<<
; 
; Macro CylinderInertiaTensorMatrix33(MatrixA, Mass, Radius, Lengthx)
;   
;   SetMatrix33Line1(MatrixA, (Mass*(3*Radius*Radius+Lengthx*Lengthx)/12), 0, 0)
;   SetMatrix33Line2(MatrixA, 0, (Mass*(3*Radius*Radius+Lengthx*Lengthx)/12), 0)
;   SetMatrix33Line3(MatrixA, 0, 0, (Mass*Radius*Radius)/2)
;   
; EndMacro

; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; <<<<< L'oprateur PrismInertiaTensorMatrix33 <<<<<
; <<<<< Le paramtre a est align avec l'axe x <<<<<
; <<<<< Le paramtre b est align avec l'axe y <<<<<
; <<<<< Le paramtre c est align avec l'axe z <<<<<

; Macro PrismInertiaTensorMatrix33(MatrixA, Mass, a, b, c)
;   
;   SetMatrix33Line1(MatrixA, (Mass*(b*b+c*c)/12), 0, 0)
;   SetMatrix33Line2(MatrixA, 0, (Mass*(a*a+c*c)/12), 0)
;   SetMatrix33Line3(MatrixA, 0, 0, (Mass*(a*a+b*b)/12))
;   
; EndMacro

; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; <<<<< L'oprateur EllipsoidInertiaTensorMatrix33 <<<<<
; <<<<< Le paramtre a est align avec l'axe x     <<<<<
; <<<<< Le paramtre b est align avec l'axe y     <<<<<
; <<<<< Le paramtre c est align avec l'axe z     <<<<<

; Macro EllipsoidInertiaTensorMatrix33(MatrixA, Mass, a, b, c)
;   
;   SetMatrix33Line1(MatrixA, (Mass*(b*b+c*c)/5), 0, 0)
;   SetMatrix33Line2(MatrixA, 0, (Mass*(a*a+c*c)/5), 0)
;   SetMatrix33Line3(MatrixA, 0, 0, (Mass*(a*a+b*b)/5))
;   
; EndMacro

; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; <<<<< L'oprateur ConeInertiaTensorMatrix33    <<<<<
; <<<<< La Hauteur est aligne selon l'axe des Z <<<<<
; <<<<< La base est est centre  l'origine.     <<<<<

; Macro ConeInertiaTensorMatrix33(MatrixA, Mass, Radius, Height)
;   
;   SetMatrix33Line1(MatrixA, (Mass*((3*Radius*Radius/20)+(3*Height*Height/5)), 0, 0)
;   SetMatrix33Line2(MatrixA, 0, (Mass*((3*Radius*Radius/20)+(3*Height*Height/5)), 0)
;   SetMatrix33Line3(MatrixA, 0, 0, (3*Mass*Radius*Radius)/10)
;   
; EndMacro

; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; <<<<< L'oprateur Star (Voir notes de cours SIGGRAPH notesd1.pdf) <<<<<

; Macro StarMatrix33(MatrixA, VectorA)
;   
;   SetMatrix33Line1(MatrixA, 0, -GetVector3k(VectorA),  GetVector3j(VectorA))
;   SetMatrix33Line2(MatrixA,  GetVector3k(VectorA), 0, -GetVector3i(VectorA))
;   SetMatrix33Line3(MatrixA, -GetVector3j(VectorA),     GetVector3i(VectorA), 0)
;   
; EndMacro

; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; <<<<< L'oprateur IsIntegerMatrix <<<<<

; Procedure.b IsIntegerMatrix33(*MatrixA.Matrix33)
;   
;   If GetMatrix33e11(*MatrixA) <> Int(GetMatrix33e11(*MatrixA))
;     ProcedureReturn #False
;   EndIf
;   
;   If GetMatrix33e21(*MatrixA) <> Int(GetMatrix33e21(*MatrixA))
;     ProcedureReturn #False
;   EndIf
;   
;   If GetMatrix33e31(*MatrixA) <> Int(GetMatrix33e31(*MatrixA))
;     ProcedureReturn #False
;   EndIf
;   
;   If GetMatrix33e12(*MatrixA) <> Int(GetMatrix33e12(*MatrixA))
;     ProcedureReturn #False
;   EndIf
;   
;   If GetMatrix33e22(*MatrixA) <> Int(GetMatrix33e22(*MatrixA))
;     ProcedureReturn #False
;   EndIf
;   
;   If GetMatrix33e32(*MatrixA) <> Int(GetMatrix33e32(*MatrixA))
;     ProcedureReturn #False
;   EndIf
;   
;   If GetMatrix33e13(*MatrixA) <> Int(GetMatrix33e13(*MatrixA))
;     ProcedureReturn #False
;   EndIf
;   
;   If GetMatrix33e23(*MatrixA) <> Int(GetMatrix33e23(*MatrixA))
;     ProcedureReturn #False
;   EndIf
;   
;   If GetMatrix33e33(*MatrixA) <> Int(GetMatrix33e33(*MatrixA))
;     ProcedureReturn #False
;   EndIf
;   
;   ProcedureReturn #True
; EndProcedure

; <<<<<<<<<<<<<<<<<<<<<<<<<<
; <<<<< FIN DU FICHIER <<<<<
; <<<<<<<<<<<<<<<<<<<<<<<<<<
; IDE Options = PureBasic 5.70 beta 1 (Linux - x64)
; Folding = 8--foP5-v6t-
; EnableXP
; CompileSourceDirectory
; EnableUnicode