; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; Nom du projet : Window - Module
; Nom du fichier : Window - Module.pb
; Version du fichier : 1.0.1
; Programmation : OK
; Programm par : Guillaume Saumure
; Alias : StarBootics
; Courriel : gsaumure@cgocable.ca
; Date : 13-02-2015
; Mise  jour : 07-04-2016
; Code PureBasic : V5.42
; Plateforme : Windows, Linux, MacOS X
; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

DeclareModule Window 
  
  Declare Initialize(P_InstanceID.l, P_PosX.l, P_PosY.l, P_Width.l, P_Height.l, P_SizeFlag.b = #True)
  Declare Reset(P_InstanceID.l)
  Declare ReadPrefs(GroupName.s, P_InstanceID.l)
  Declare WritePrefs(GroupName.s, P_InstanceID.l)
  Declare Open(P_InstanceID.l, Flag = #PB_Window_SystemMenu)
  Declare OpenEx(P_InstanceID.l, Title.s, Flag = #PB_Window_SystemMenu, ParentID = #Null)
  Declare Reshape(P_InstanceID.l)
  Declare GetCurrentPosition(P_InstanceID.l)
  Declare GetCurrentSize(P_InstanceID.l)
  Declare GetCurrentPositionSize(P_InstanceID.l)
  Declare SanityCheck(P_InstanceID.l)
  Declare.l Width(P_InstanceID.l)
  Declare.l Height(P_InstanceID.l)
  Declare SetBounds(P_InstanceID.l, P_MinWidth.l, P_MinHeight.l, P_MaxWidth.l = #PB_Ignore, P_MaxHeight.l = #PB_Ignore)

EndDeclareModule

Module Window
  
  Structure Instance
    ID.l
    PosX.l
    PosY.l
    Width.l
    Height.l
    SizeFlag.b
    BoundsFlag.l
    MinWidth.l
    MinHeight.l
    MaxWidth.l
    MaxHeight.l
  EndStructure
 
  Global NewList Instances.Instance()
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Procdures prives <<<<<
  
  Procedure SearchInstances(P_InstanceID.l)
    
    Protected IsFoundAtIndex.l = -1
    
    ForEach Instances()
      
      If Instances()\ID = P_InstanceID
        IsFoundAtIndex = ListIndex(Instances())
        Break
      EndIf
      
    Next
    
    ProcedureReturn IsFoundAtIndex
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Procdures publiques <<<<<
  
  Procedure Initialize(P_InstanceID.l, P_PosX.l, P_PosY.l, P_Width.l, P_Height.l, P_SizeFlag.b = #True)
    
    If SearchInstances(P_InstanceID) = -1
      AddElement(Instances())
    EndIf
    
    Instances()\ID = P_InstanceID
    Instances()\PosX = P_PosX
    Instances()\PosY = P_PosY 
    Instances()\Width = P_Width
    Instances()\Height = P_Height
    Instances()\SizeFlag = P_SizeFlag
    
    If Instances()\BoundsFlag = 0
      Instances()\BoundsFlag = #PB_Ignore
      Instances()\MinWidth = #PB_Ignore
      Instances()\MinHeight = #PB_Ignore
      Instances()\MaxWidth = #PB_Ignore
      Instances()\MaxHeight = #PB_Ignore
    EndIf
    
  EndProcedure
  
  Procedure Reset(P_InstanceID.l)
    
    If P_InstanceID = -1
      
      ForEach Instances()
        Instances()\ID = 0
        Instances()\PosX = 0
        Instances()\PosY = 0
        Instances()\Width = 0
        Instances()\Height = 0
        Instances()\SizeFlag = 0
        Instances()\BoundsFlag = 0
        Instances()\MinWidth = 0
        Instances()\MinHeight = 0
        Instances()\MaxWidth = 0
        Instances()\MaxHeight = 0
      Next
      
      ClearList(Instances())
      
    Else
      
      If SearchInstances(P_InstanceID) <> -1
        Instances()\ID = 0
        Instances()\PosX = 0
        Instances()\PosY = 0
        Instances()\Width = 0
        Instances()\Height = 0
        Instances()\SizeFlag = 0
        Instances()\BoundsFlag = 0
        Instances()\MinWidth = 0
        Instances()\MinHeight = 0
        Instances()\MaxWidth = 0
        Instances()\MaxHeight = 0
        DeleteElement(Instances(), 1)
      EndIf

    EndIf
    
  EndProcedure
  
  Procedure ReadPrefs(GroupName.s, P_InstanceID.l)
    
    If SearchInstances(P_InstanceID) <> -1
      
      PreferenceGroup(GroupName)
      
      Instances()\PosX = ReadPreferenceLong("PosX", Instances()\PosX)
      Instances()\PosY = ReadPreferenceLong("PosY", Instances()\PosY)
      
      If Instances()\SizeFlag = #True
        Instances()\Width = ReadPreferenceLong("Width", Instances()\Width)
        Instances()\Height = ReadPreferenceLong("Height", Instances()\Height)
      EndIf
      
      If Instances()\BoundsFlag = #True
        
        If Instances()\MinWidth <> #PB_Ignore 
          Instances()\MinWidth = ReadPreferenceLong("MinWidth", Instances()\MinWidth)
        EndIf
        
        If Instances()\MinHeight <> #PB_Ignore 
          Instances()\MinHeight = ReadPreferenceLong("MinHeight", Instances()\MinHeight)
        EndIf
        
        If Instances()\MaxWidth <> #PB_Ignore 
          Instances()\MaxWidth = ReadPreferenceLong("MaxWidth", Instances()\MaxWidth)
        EndIf
        
        If Instances()\MaxHeight <> #PB_Ignore 
          Instances()\MaxHeight = ReadPreferenceLong("MaxHeight", Instances()\MaxHeight)
        EndIf
        
      EndIf
      
    EndIf
    
  EndProcedure
  
  Procedure WritePrefs(GroupName.s, P_InstanceID.l)
    
    If SearchInstances(P_InstanceID) <> -1
      
      PreferenceGroup(GroupName)
      
      WritePreferenceLong("PosX", Instances()\PosX)
      WritePreferenceLong("PosY", Instances()\PosY)
      
      If Instances()\SizeFlag = #True
        WritePreferenceLong("Width", Instances()\Width)
        WritePreferenceLong("Height", Instances()\Height)
      EndIf
      
      If Instances()\BoundsFlag = #True
        
        If Instances()\MinWidth <> #PB_Ignore 
          WritePreferenceLong("MinWidth", Instances()\MinWidth)
        EndIf
        
        If Instances()\MinHeight <> #PB_Ignore 
          WritePreferenceLong("MinHeight", Instances()\MinHeight)
        EndIf
        
        If Instances()\MaxWidth <> #PB_Ignore 
          WritePreferenceLong("MaxWidth", Instances()\MaxWidth)
        EndIf
        
        If Instances()\MaxHeight <> #PB_Ignore 
          WritePreferenceLong("MaxHeight", Instances()\MaxHeight)
        EndIf
        
      EndIf
      
    EndIf
    
  EndProcedure
  
  Procedure Open(P_InstanceID.l, Flag = #PB_Window_SystemMenu)
    
    If SearchInstances(P_InstanceID) <> -1
      
      Success = OpenWindow(P_InstanceID, Instances()\PosX, Instances()\PosY, Instances()\Width, Instances()\Height, "", Flag)
      
      If Instances()\BoundsFlag = #True
        WindowBounds(P_InstanceID, Instances()\MinWidth, Instances()\MinHeight, Instances()\MaxWidth, Instances()\MaxHeight)
      EndIf
      
    EndIf
    
    ProcedureReturn Success
  EndProcedure
  
  Procedure OpenEx(P_InstanceID.l, Title.s, Flag = #PB_Window_SystemMenu, ParentID = #Null)
    
    If SearchInstances(P_InstanceID) <> -1
      
      Success = OpenWindow(P_InstanceID, Instances()\PosX, Instances()\PosY, Instances()\Width, Instances()\Height, Title, Flag, ParentID)
      
      If Instances()\BoundsFlag = #True
        WindowBounds(P_InstanceID, Instances()\MinWidth, Instances()\MinHeight, Instances()\MaxWidth, Instances()\MaxHeight)
      EndIf
      
    EndIf
    
    ProcedureReturn Success
  EndProcedure
  
  Procedure Reshape(P_InstanceID.l)
    
    If SearchInstances(P_InstanceID) <> -1
      ResizeWindow(P_InstanceID, Instances()\PosX, Instances()\PosY, Instances()\Width, Instances()\Height)
    EndIf
    
  EndProcedure 
  
  Procedure GetCurrentPosition(P_InstanceID.l)
    
    If SearchInstances(P_InstanceID) <> -1
      Instances()\PosX = WindowX(P_InstanceID)
      Instances()\PosY = WindowY(P_InstanceID)
    EndIf
    
  EndProcedure
  
  Procedure GetCurrentSize(P_InstanceID.l)
    
    If SearchInstances(P_InstanceID) <> -1
      Instances()\Width = WindowWidth(P_InstanceID)
      Instances()\Height = WindowHeight(P_InstanceID)
    EndIf
    
  EndProcedure
  
  Procedure GetCurrentPositionSize(P_InstanceID.l)
    
    If SearchInstances(P_InstanceID) <> -1
      Instances()\PosX = WindowX(P_InstanceID)
      Instances()\PosY = WindowY(P_InstanceID)
      Instances()\Width = WindowWidth(P_InstanceID)
      Instances()\Height = WindowHeight(P_InstanceID)
    EndIf
    
  EndProcedure
  
  Procedure SanityCheck(P_InstanceID.l)
    
    If SearchInstances(P_InstanceID) <> -1
      
      If ExamineDesktops()
        
        If Instances()\PosX > DesktopWidth(0) Or Instances()\PosX < 0
          Instances()\PosX = 50
        EndIf
        
        If Instances()\PosY > DesktopHeight(0) Or Instances()\PosY < 0
          Instances()\PosY = 50
        EndIf
        
        If (Instances()\PosX + Instances()\Width) > DesktopWidth(0)
          Instances()\PosX = Instances()\PosX - Instances()\Width
        EndIf
        
        If (Instances()\PosY + Instances()\Height) > DesktopHeight(0)
         Instances()\PosY = Instances()\PosY - Instances()\Height
        EndIf
        
        If Instances()\SizeFlag = #True
          
          If Instances()\Width > DesktopWidth(0)
            Instances()\Width = DesktopWidth(0) - Instances()\PosX
          EndIf
          
          If Instances()\Height > DesktopHeight(0)
            Instances()\Height = DesktopHeight(0) - Instances()\PosY
          EndIf 
          
        EndIf
        
      Else 
        
        Instances()\PosX = 50
        Instances()\PosY = 50
        
      EndIf
      
    EndIf
    
  EndProcedure
  
  Procedure.l Width(P_InstanceID.l)
    
    If SearchInstances(P_InstanceID) <> -1
      ProcedureReturn Instances()\Width
    EndIf
    
  EndProcedure
  
  Procedure.l Height(P_InstanceID.l)
    
    If SearchInstances(P_InstanceID) <> -1
      ProcedureReturn Instances()\Height
    EndIf
    
  EndProcedure
  
  Procedure SetBounds(P_InstanceID.l, P_MinWidth.l, P_MinHeight.l, P_MaxWidth.l = #PB_Ignore, P_MaxHeight.l = #PB_Ignore)
    
    If SearchInstances(P_InstanceID) <> -1
      Instances()\BoundsFlag = #True
      Instances()\MinWidth = P_MinWidth
      Instances()\MinHeight = P_MinHeight
      Instances()\MaxWidth = P_MaxWidth
      Instances()\MaxHeight = P_MaxHeight
    EndIf
    
  EndProcedure
  
EndModule

; <<<<<<<<<<<<<<<<<<<<<<<<<<
; <<<<< FIN DU FICHIER <<<<<
; <<<<<<<<<<<<<<<<<<<<<<<<<<
; IDE Options = PureBasic 5.70 beta 1 (Linux - x64)
; Folding = ----
; EnableXP
; CompileSourceDirectory